import React from 'react';
import * as BS from 'react-bootstrap';
import _ from 'lodash';
import Multiselect from 'react-bootstrap-multiselect';

class DefaultProductVersion extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      lpSelectedMilestones: [window.commonStore.lp_milestone_in_focus],
      trSelectedMilestones: [window.commonStore.tr_milestone_in_focus],
      dashboardSelectedMilestones: [window.commonStore.dashboard_in_focus.name],
    };
    this.handleMilestoneSelectionLp = this.handleMilestoneSelectionLp.bind(this);
    this.handleMilestoneSelectionTr = this.handleMilestoneSelectionTr.bind(this);
    this.handleDashboardSelection = this.handleDashboardSelection.bind(this);
    this.setLpMilestone = this.setLpMilestone.bind(this);
    this.setTrMilestone = this.setTrMilestone.bind(this);
    this.setDashboardMilestone = this.setDashboardMilestone.bind(this);
  }

  handleMilestoneSelectionLp() {
    const lpSelectedMilestones = [];
    this.milestonesSelector.$multiselect.find(':selected').each((i, elem) => {
      lpSelectedMilestones.push($(elem).val());
    });
    this.setState({ lpSelectedMilestones });
  }

  handleMilestoneSelectionTr() {
    const trSelectedMilestones = [];
    this.milestonesSelectorTr.$multiselect.find(':selected').each((i, elem) => {
      trSelectedMilestones.push($(elem).val());
    });
    this.setState({ trSelectedMilestones });
  }

  handleDashboardSelection() {
    const dashboardSelectedMilestones = [];
    this.milestonesSelectorDashboard.$multiselect.find(':selected').each((i, elem) => {
      dashboardSelectedMilestones.push($(elem).val());
    });
    this.setState({ dashboardSelectedMilestones });
  }


  setLpMilestone() {
    $.getJSON('/admin/_set_lp_milestone', {
      milestone: $('#lp option:selected').val() });
    return false;
  }

  setTrMilestone() {
    $.getJSON('/admin/_set_tr_milestone', {
      milestone: $('#tr option:selected').val() });
    return false;
  }

  setDashboardMilestone() {
    $.getJSON('/admin/_set_dashboard_milestone', {
      milestone: $('#dashboard option:selected').val() });
    return false;
  }

  render() {
    const lpMilestones = _.map(window.commonStore.launchpad_milestones, (v) => ({
      value: v, selected: _.includes(this.state.lpSelectedMilestones[0], v),
    }));
    const milestonesSelectLp = (
      <Multiselect
        id={'lp'}
        key="milestoneLp.select"
        ref={(c) => { this.milestonesSelector = c; }}
        buttonWidth="250px"
        maxHeight={300}
        enableFiltering
        nonSelectedText="select milestone"
        allSelectedText="all selected"
        onDropdownHide={this.handleMilestoneSelectionLp}
        data={lpMilestones}
      />
      );
    const trMilestone = window.commonStore.testrail_milestones.map(m => ({
      value: m.name,
      selected: _.includes(this.state.trSelectedMilestones[0], m.name),
    }));

    const milestonesSelectTr = (
      <Multiselect
        id={'tr'}
        key="milestoneTr.select"
        ref={(c) => { this.milestonesSelectorTr = c; }}
        buttonWidth="250px"
        maxHeight={300}
        enableFiltering
        nonSelectedText="select milestone"
        allSelectedText="all selected"
        onDropdownHide={this.handleMilestoneSelectionTr}
        data={trMilestone}
      />
      );

    const releaseData = _.intersection(
      _.map(window.commonStore.testrail_milestones, 'name'),
      window.commonStore.launchpad_milestones
    ).map(v => ({ value: v, selected: v === this.state.dashboardSelectedMilestones[0] }));

    const milestonesSelectDashboard = (
      <Multiselect
        id={'dashboard'}
        key="dashboard.select"
        ref={(c) => { this.milestonesSelectorDashboard = c; }}
        buttonWidth="250px"
        maxHeight={300}
        enableFiltering
        nonSelectedText="select milestone"
        allSelectedText="all selected"
        onDropdownHide={this.handleDashboardSelection}
        data={releaseData}
      />
      );

    return (
          <div style={{ width: "40%" }}>
            <table className="table table-bordered">
              <tbody>
                <tr>
                  <td
                    style={{ verticalAlign: 'middle' }}
                    className="field-label col-md-2"
                  ><label>LaunchPad</label></td>
                  <td style={{ verticalAlign: 'middle' }} className="col-md-5">
                    {milestonesSelectLp}
                  </td>
                  <td
                    style={{ position: 'relative' }}
                    className="col-md-5"
                  ><BS.Button
                    style={{ position: 'absolute', left: '50%', top: '50%', margin: '-20px -45px' }}
                    className="btn btn-raised btn-success"
                    onClick={this.setLpMilestone}>
                    Set
                  </BS.Button></td>
                </tr>
                <tr>
                  <td
                    style={{ verticalAlign: 'middle' }}
                    className="field-label col-md-2"
                  ><label>Testrail</label></td>
                  <td style={{ verticalAlign: 'middle' }} className="col-md-4">
                    {milestonesSelectTr}
                  </td>
                  <td
                    style={{ position: 'relative' }}
                    className="col-md-4"
                  ><BS.Button
                    style={{ position: 'absolute', left: '50%', top: '50%', margin: '-20px -45px' }}
                    className="btn btn-raised btn-success"
                    onClick={this.setTrMilestone}>
                    Set
                  </BS.Button></td>
                </tr>
                <tr>
                  <td
                    style={{ verticalAlign: 'middle' }}
                    className="field-label col-md-2"
                  ><label>Dashboard</label></td>
                  <td style={{ verticalAlign: 'middle' }} className="col-md-4">
                    {milestonesSelectDashboard}
                  </td>
                  <td
                    style={{ position: 'relative' }}
                    className="col-md-4"
                  ><BS.Button
                    style={{ position: 'absolute', left: '50%', top: '50%', margin: '-20px -45px' }}
                    className="btn btn-raised btn-success"
                    onClick={this.setDashboardMilestone}>
                    Set
                  </BS.Button></td>
                </tr>
              </tbody>
            </table>
          </div>
    );
  }
}

export default DefaultProductVersion;
