import React, { Component } from 'react';
import update from 'react/lib/update';
import { DragDropContext } from 'react-dnd';
import HTML5Backend from 'react-dnd-html5-backend';
import * as BS from 'react-bootstrap';
import _ from 'lodash';

import FilterRow from './components/FilterRow';

const style = {
  width: '40%',
};

@DragDropContext(HTML5Backend)
export default class FeatureTestingPresets extends Component {
  constructor(props) {
    super(props);
    this.moveFilter = this.moveFilter.bind(this);
    this.handleAddRow = this.handleAddRow.bind(this);
    this.handleDelRow = this.handleDelRow.bind(this);
    this.onSaveFilter = this.onSaveFilter.bind(this);
    this.state = {
      filters: window.commonStore.jira_filter_ids,
    };
  }

  handleAddRow(e) {
    const id = (+ new Date() + Math.floor(Math.random() * 939999)).toString(36);
    const position = this.state.filters.length;
    const _filter = {
      name: '',
      id,
      position,
    };
    this.state.filters.push(_filter);
    this.setState(this.state.filters);
  }

  handleDelRow(_filter) {
    $.ajax({
      url: '/admin/_remove_filter',
      dataType: 'json',
      type: 'POST',
      data: _filter,
      success: function(data) {
        this.setState({ filters: data });
      }.bind(this),
      error: function(xhr, status, err) {
        console.error(status, err.toString());
      }.bind(this),
    });
  }

  onSaveFilter(_filter) {
    $.ajax({
      url: '/admin/_save_filter',
      dataType: 'json',
      type: 'POST',
      data: _filter,
      success: function(data) {
        this.setState({ filters: data });
      }.bind(this),
      error: function(xhr, status, err) {
        console.error(status, err.toString());
      }.bind(this),
    });
  }

  moveFilter(dragIndex, hoverIndex) {
    const { filters } = this.state;

    const dragFilter = filters[dragIndex];

    const filtersState = update(this.state, {
      filters: {
        $splice: [
         [dragIndex, 1],
         [hoverIndex, 0, dragFilter],
        ],
      },
    });

    filtersState.filters.forEach((elem, index) => {
      elem.position = index;
    }
  );
    this.setState(filtersState);
  }

  render() {
    const { filters } = this.state;

    return (
      <div style={style}>
        <table className="table table-condensed">
          <tbody>
            {_(filters).sortBy('position').map((_filter, i) => {
              return (
                <FilterRow
                  key={_filter.id}
                  id={_filter.id}
                  index={i}
                  link={_filter.id}
                  name={_filter.name}
                  position={_filter.position}
                  moveFilter={this.moveFilter}
                  onDelEvent={this.handleDelRow}
                  onSaveFilter={this.onSaveFilter}
                />
              );
            }).value()}
          </tbody>
        </table>
        <BS.Button className="btn btn-raised btn-success" onClick={this.handleAddRow}>
          Add
        </BS.Button>
      </div>
     );
  }
 }
