import React from 'react';
import * as BS from 'react-bootstrap';
import _ from 'lodash';
import $ from 'jquery';
import Multiselect from 'react-bootstrap-multiselect';
import moment from 'moment';

import DatePicker from './DatePicker';
import CheckBox from '../../components/CheckBox';

class MilestoneBox extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      selectedTeam: [],
      selectedType: [],
      allRuns: [],
      selectedRuns: [],
      runList: [],
      is_any: false,
    };
    this.handleTeamSelection = this.handleTeamSelection.bind(this);
    this.handleTypeSelection = this.handleTypeSelection.bind(this);
    this.handleRunSelection = this.handleRunSelection.bind(this);
    this.handleAddRun = this.handleAddRun.bind(this);
    this.handeleDeleteRun = this.handeleDeleteRun.bind(this);
    this.onGetData = this.onGetData.bind(this);
    this.handleSetStartDate = this.handleSetStartDate.bind(this);
    this.onToggleAnyCheckBox = this.onToggleAnyCheckBox.bind(this);
  }

  componentDidMount() {
    this.onMount();
  }

  onMount() {
    this.setState({ runList: this.props.data.runs });
    this.setState({ is_any: this.props.data.is_any });
  }

  getRunNames() {
    const teamId = this.state.selectedTeam[0];
    const typeId = this.state.selectedType;
    if (!teamId || !typeId.length) {
      this.setState({ allRuns: [] });
      return;
    }
    const types = typeId.map(t => `&run_types=${t}`).join('');
    const url = `/api/v1.0/testrail/run_list?milestone_id=${this.props.data.milestone_id}&qa_team=${teamId}${types}`;
    $.get(url)
      .done(this.updateRuns.bind(this));
  }

  onGetData(data) {
    this.setState({ runList: data });
  }

  handleAddRun() {
    const runs = this.state.runList;
    const team = parseInt(this.state.selectedTeam, 10);
    const index = _.findIndex(runs, obj => obj.qa_team_id === team);

    if (index !== -1) {
      const iniCount = runs[index].run_names.length;
      _.each(this.state.selectedRuns, (run) => {
        if (!_.includes(runs[index].run_names, run)) {
          runs[index].run_names.push(run);
        }
      });
      if (iniCount !== runs[index].run_names.length) {
        this.setState({ runList: runs });
      }
    } else {
      runs.push(
        {
          qa_team_id: team,
          run_names: this.state.selectedRuns,
        }
      );
      this.setState({ runList: runs });
    }

    $.ajax({
      url: '/admin/_add_acceptance_run',
      dataType: 'json',
      type: 'POST',
      data: { milestone_id: this.props.data.milestone_id, run_names: this.state.selectedRuns },
      error: (xhr, status, err) =>
        console.error(status, err.toString())
      .bind(this),
    });
  }

  handeleDeleteRun() {
    const runs = this.state.runList;
    const team = parseInt(this.state.selectedTeam, 10);
    const index = _.findIndex(runs, obj => obj.qa_team_id === team);
    _.remove(runs[index].run_names, name => _.includes(this.state.selectedRuns, name));
    if (!runs[index].run_names.length) {
      _.remove(runs, obj => obj.qa_team_id === team);
    }
    this.setState({ runList: runs });
    $.ajax({
      url: '/admin/_remove_acceptance_run',
      dataType: 'json',
      type: 'POST',
      data: { milestone_id: this.props.data.milestone_id, run_names: this.state.selectedRuns },
      error: (xhr, status, err) =>
        console.error(status, err.toString())
      .bind(this),
    });
  }

  handleSetStartDate(startDate, teamId) {
    const data = {
      milestone_id: this.props.data.milestone_id,
      team_id: teamId,
      start_date: startDate.format(),
    };
    $.ajax({
      url: '/admin/_set_acceptance_date',
      dataType: 'json',
      type: 'POST',
      data,
      error: (xhr, status, err) =>
        console.error(status, err.toString()).bind(this),
    });
  }

  updateRuns(data) {
    this.setState({ allRuns: data });
  }

  handleRunSelection() {
    const selectedRuns = [];
    this.runSelector.$multiselect.find(':selected').each((i, elem) => {
      selectedRuns.push($(elem).val());
    });
    this.setState({ selectedRuns });
  }

  handleTeamSelection() {
    const selectedTeam = [];
    this.teamSelector.$multiselect.find(':selected').each((i, elem) => {
      selectedTeam.push($(elem).val());
    });
    this.setState({ selectedTeam });
  }

  handleTypeSelection() {
    const selectedType = [];
    this.typeSelector.$multiselect.find(':selected').each((i, elem) => {
      selectedType.push($(elem).val());
    });
    this.setState({ selectedType });
    this.getRunNames();
  }

  onToggleAnyCheckBox(e) {
    $.ajax({
      url: '/admin/_set_acceptance_any',
      dataType: 'json',
      type: 'POST',
      data: { milestone_id: this.props.data.milestone_id, value: e.target.checked },
      error: (xhr, status, err) =>
        console.error(status, err.toString()).bind(this),
    });
    this.setState({ is_any: e.target.checked });
  }


  render() {
    const teamSelector = window.commonStore.testrail_qa_teams.map(team => ({
      label: team.name,
      value: team.id,
      selected: _.includes(this.state.selectedTeam[0], team.id),
    }));

    const teamSelect = (
      <Multiselect
        id={'team'}
        key="team.select"
        ref={(c) => { this.teamSelector = c; }}
        buttonWidth="195px"
        maxHeight={300}
        enableFiltering
        nonSelectedText="select team"
        onDropdownHide={this.handleTeamSelection}
        data={teamSelector}
      />
    );

    const typeSelector = window.commonStore.testrail_run_types.map(type => ({
      label: type.name,
      value: type.id,
      selected: _.includes(this.state.selectedType, type.id.toString()),
    }));

    const typeSelect = (
      <Multiselect
        id={'type'}
        key="type.select"
        buttonWidth="195px"
        ref={(c) => { this.typeSelector = c; }}
        multiple
        maxHeight={300}
        includeSelectAllOption
        enableFiltering
        nonSelectedText="select type"
        allSelectedText="all selected"
        onDropdownHide={this.handleTypeSelection}
        data={typeSelector}
      />
    );

    const runSelect = this.state.allRuns.map(run => ({
      label: run,
      value: run,
      selected: _.includes(this.state.selectedRuns, run),
    }));

    const runsSelect = (
      <Multiselect
        id={'runs'}
        key="runs.select"
        buttonWidth="195px"
        ref={(c) => { this.runSelector = c; }}
        multiple
        includeSelectAllOption
        maxHeight={300}
        disableIfEmpty
        enableFiltering
        allSelectedText="all selected"
        onDropdownHide={this.handleRunSelection}
        data={runSelect}
      />
    );

    const checkBox = (
      <CheckBox
        label=" Any"
        key="any"
        checked={this.state.is_any}
        onChange={this.onToggleAnyCheckBox}
        groupClassName="checkbox-inline"
        togglebutton
      />
    );

    return (
      <div>
        <RunList data={this.state.runList} onChange={this.handleSetStartDate} />
        <BS.ButtonToolbar>
          {teamSelect}
          {typeSelect}
          {runsSelect}
          {checkBox}
          <BS.Button bsStyle="success" onClick={this.handleAddRun}>Add</BS.Button>
          <BS.Button bsStyle="danger" onClick={this.handeleDeleteRun}>Delete</BS.Button>
        </BS.ButtonToolbar>
      </div>
    );
  }
}


const RunList = (props) => {
  const listByTeam = (
    <div>
      {
        _.map(
        props.data,
        (team, index) => {
          const teamInfo = window.commonStore.testrail_qa_teams.filter(obj => obj.id === team.qa_team_id);
          const runList = team.run_names.map((name, index) => {
            return (
              <BS.ListGroupItem key={index}>{name}</BS.ListGroupItem>
            );
          });
          return (
            <div key={index} className="panel panel-default">
              <div className="panel-heading">
                <div className="panel-title pull-left" role="toolbar" style={{ fontSize: '16px', paddingBottom: '7px', paddingTop: '7px' }}>
                  {teamInfo[0].name}
                </div>
                <div className="panel-title pull-right" style={{ fontSize: '16px' }}>
                  <DatePicker
                    style={{ width: '120px', marginTop: '0px', marginBottom: '0px' }}
                    teamId={team.qa_team_id}
                    startDate={moment(team.date)}
                    onChange={props.onChange}
                  />
                </div>
                <div className="clearfix" />
              </div>
              <ul className="list-group">
                {runList}
              </ul>
            </div>
          );
        }
        )
      }
    </div>
  );
  return (
    <div>
      {listByTeam}

    </div>
  );
};


export default MilestoneBox;
