/**
 * Created by farhad on 02.06.16.
 */
import React from 'react';
import ReactDOM from 'react-dom';
import _ from 'lodash';
import { AgGridReact } from 'ag-grid-react';


export default class AgGridExtended extends React.Component {

  componentDidMount(){
    this.setupShiftSelection();
  }

  setupShiftSelection(){
    let currSelectedRowIndex, prevSelectedRowIndex, lastCellClickParams, lastSelectedNodes;

    this.gridCmp.api.addEventListener('cellClicked', (params) => {
      lastCellClickParams = params;
      lastSelectedNodes = this.gridCmp.api.getSelectedNodes();
    });

    this.gridCmp.api.addEventListener('rowSelected', ()=>{
      const focusedCell = this.gridCmp.api.getFocusedCell();
      if(focusedCell){
        prevSelectedRowIndex = currSelectedRowIndex;
        currSelectedRowIndex = focusedCell.rowIndex;
      }
      if(currSelectedRowIndex !== undefined &&
         prevSelectedRowIndex !== undefined &&
         lastCellClickParams &&  lastCellClickParams.rowIndex === currSelectedRowIndex &&
         lastCellClickParams.event.shiftKey
      ){
        _.each(lastSelectedNodes, (node)=>{
          this.gridCmp.api.selectNode(node, true, true);
        });

        const [index1, index2] = _.sortBy([prevSelectedRowIndex, currSelectedRowIndex]);
        for(let i=index1; i<=index2; i++){
          this.gridCmp.api.selectIndex(i, true, true);
        }
      }
    });
  }

  render() {
    const {gridOptions, ...restProps} = this.props;

    return (
      <div
        className="ag-bootstrap"
        {...restProps}
      >
        <AgGridReact
          ref={(cmp) => this.gridCmp = cmp}
          {...gridOptions}
        />
      </div>
    )
  }
}
