/**
 * Created by farhad on 01.06.16.
 */
import React from 'react';
import _ from 'lodash';

const MODE_INCLUDE = 'include';
const MODE_EXCLUDE= 'exclude';
const DEFAULT_DELIMITER = ',';


export default class extends React.Component {

  constructor() {
    super();

    this.state = {
      content: '',
      mode: MODE_INCLUDE,
    };

    this.valuesMap = null;

    this.api = {
      getModel: this.apiGetModel.bind(this),
      setModel: this.apiSetModel.bind(this),
    };

    this.onFilterModeChange = this.onFilterModeChange.bind(this);
    this.onFilterContentChange = this.onFilterContentChange.bind(this);
    this.onFilterChange = this.onFilterChange.bind(this);
    this.onDebouncedFilterChange = _.debounce(this.onDebouncedFilterChange.bind(this), 500);
  }

  // called by agGrid
  init(params) {
    this.filterChangedCallback = params.filterChangedCallback;
    this.filterModifiedCallback = params.filterModifiedCallback;

    this.valueGetter = params.valueGetter;
    const filterParams = {
      mode: MODE_INCLUDE,
      delimiterDisplay: DEFAULT_DELIMITER,
      delimiter: DEFAULT_DELIMITER,
      ...(params.filterParams || {})
    };

    this.delimiter = filterParams.delimiter;
    this.delimiterDisplay = filterParams.delimiterDisplay;
    this.setState({mode: filterParams.mode})
  }

  populateValuesMap(){
    this.valuesMap = Object.create(null);
    this.state.content.split(this.delimiter).forEach((v) => {
      const key = v.trim();
      if(key){
        this.valuesMap[key] = true;
      }
    })
  }

  // called by agGrid
  isFilterActive() {
    return !!this.state.content;
  };

  // called by agGrid
  doesFilterPass(params) {
    if (this.valuesMap === null) {
      this.populateValuesMap()
    }
    const value = this.valueGetter(params);
    const isPresent = this.valuesMap[value] === true;

    if (this.state.mode === MODE_INCLUDE)
      return isPresent;

    if (this.state.mode === MODE_EXCLUDE)
      return !isPresent;

    console.warn('invalid filter mode ' + this.state.mode)
  };

  onFilterChange(){
    this.filterModifiedCallback();
    this.filterChangedCallback();
  }

  onDebouncedFilterChange(){
    this.onFilterChange();
  }

  onFilterModeChange(event) {
    this.setState({mode: event.target.value}, this.onFilterChange);
  }

  onFilterContentChange(event) {
    this.valuesMap = null;
    this.setState({content: event.target.value}, this.onDebouncedFilterChange);
  }

  apiGetModel(){
    if(!this.isFilterActive()){
      return null;
    }
    return {
      mode: this.state.mode,
      content: this.state.content,
    }
  }

  apiSetModel(dataModel){
    const model = dataModel || {content: '', mode: this.state.mode};

    this.setState({
      content: model.content,
      mode: model.mode,
    });

    // We also must set state directly, because this method
    // should change filter immediately
    this.state.content = model.content;
    this.state.mode = model.mode;

    this.valuesMap = null;
  }

  getApi() {
    return this.api;
  }

  render() {

    const modePicker = (
      <select
        value={this.state.mode}
        onChange={this.onFilterModeChange}
        style={{width: '100%'}}
      >
        <option value={MODE_INCLUDE}>Include</option>
        <option value={MODE_EXCLUDE}>Exclude</option>
      </select>
    );

    return (
      <div>
        {modePicker}
        <div>
          <textarea
            style={{minHeight: 150}}
            placeholder={`Specify values. Use "${this.delimiterDisplay}" as delimiter`}
            value={this.state.content}
            onChange={this.onFilterContentChange}
          />
        </div>
      </div>
    );
  }
}
