/**
 * Created by farhad on 03.03.16.
 */
import React from 'react';
import _ from 'lodash';
import { FormGroup, FormControl, InputGroup } from 'react-bootstrap';
import DateRangePicker from 'react-bootstrap-daterangepicker';
import moment from 'moment';


export default class SimpleRangePicker extends React.Component {

  static propTypes = {
    onChange: React.PropTypes.func,
    startDate: React.PropTypes.object,
  };

  constructor(props) {
    super(props);
    this.state = {
      startDate: this.props.startDate || moment().subtract(6, 'days'),
      endDate: this.props.endDate || moment(),
    };
    this.handleEvent = this.handleEvent.bind(this);
  }

  getDefaultPresetRanges() {
    return {
      Today: [
        moment().startOf('day'),
        moment().endOf('day')
      ],
      Yesterday: [
        moment().subtract(1, 'days').startOf('day'),
        moment().subtract(1, 'days').endOf('day')
      ],
      'Last 7 Days': [
        moment().subtract(6, 'days').startOf('day'),
        moment().endOf('day')
      ],
      'Last 30 Days': [
        moment().subtract(29, 'days').startOf('day'),
        moment().endOf('day')
      ],
      'This Month': [
        moment().startOf('month'),
        moment().endOf('month')
      ],
      'Last Month': [
        moment().subtract(1, 'month').startOf('month'),
        moment().subtract(1, 'month').endOf('month'),
      ],
    }
  }

  componentWillReceiveProps(nextProps){
    const upd = {};
    if(nextProps.startDate){
      upd.startDate = moment(nextProps.startDate)
    }
    if(nextProps.endDate){
      upd.endDate = moment(nextProps.endDate)
    }
    this.setState(upd)
  }

  handleEvent(event, picker) {
    const startDate = moment(picker.startDate);
    const endDate = moment(picker.endDate);

    if (startDate.isSame(this.state.startDate) && endDate.isSame(this.state.endDate)) {
      return;
    }

    this.setState({
      startDate: startDate,
      endDate: endDate,
    });


    this.props.onChange([moment(startDate), moment(endDate)]);
  }

  render() {
    const start = this.state.startDate.format('YYYY-MM-DD');
    const end = this.state.endDate.format('YYYY-MM-DD');
    const currentLabel = (start === end) ? end : `${start} - ${end}`;

      return (
      <DateRangePicker
        className="simple-range-picker"
        startDate={this.state.startDate}
        endDate={this.state.endDate}
        onEvent={this.handleEvent}
        opens="center"
        ranges={this.getDefaultPresetRanges()}
        {..._.omit(this.props, 'onChange')}
      >
        <FormGroup>
          <InputGroup>
            <InputGroup.Addon><span> Dates</span></InputGroup.Addon>
            <FormControl type="text" value={currentLabel} onChange={() => {}} />
            <InputGroup.Addon><i className="fa fa-calendar" /></InputGroup.Addon>
          </InputGroup>
        </FormGroup>
      </DateRangePicker>
    );
  }


}
