import _ from 'lodash';
import React from 'react';
import * as BS from 'react-bootstrap';

import { safeDiv } from '../../reports/base/utils';
import ProgressBarLabel from './ProgressBarLabel';
import StatusPopover from './StatusPopover';
import ProgressBarTitle from './ProgressBarTitle';

const ProgressBar = ({ data, max, name }) => (
  <div style={{ position: 'relative' }}>
    <div style={{ position: 'absolute', left: '50%', transform: 'translateX(-50%)', WebkitTransform: 'translateX(-50%)' }}>
      <ProgressBarLabel
        percentage={`${safeDiv(data.passed * 100, max).toFixed(1)}%`}
        part={data.passed}
        total={max}
        ending={'cases passed'}
      />
    </div>
    <ProgressBarTitle name={name} />
    <StatusPopover counts={data}>
      <BS.ProgressBar style={{ height: 100, marginBottom: 5 }}>
        <BS.ProgressBar
          style={{ height: 100 }}
          max={max}
          key={'passed'}
          now={data.passed}
          bsStyle="success"
        />
        {_.map(_.omitBy(data, (val, key) => key === 'passed' || val === 0), (val, key) => (
          <BS.ProgressBar
            style={{ height: 100 }}
            max={max}
            key={key}
            now={val}
            bsClass={`progress-bar progress-bar-${key}`}
          />))
      }
      </BS.ProgressBar>
    </StatusPopover>
  </div>
);

ProgressBar.propTypes = {
  data: React.PropTypes.shape({
    blocked: React.PropTypes.number,
    failed: React.PropTypes.number,
    fixed: React.PropTypes.number,
    in_progress: React.PropTypes.number,
    infra_failed: React.PropTypes.number,
    other: React.PropTypes.number,
    passed: React.PropTypes.number,
    prod_failed: React.PropTypes.number,
    regression: React.PropTypes.number,
    skipped: React.PropTypes.number,
    test_failed: React.PropTypes.number,
    untested: React.PropTypes.number,
  }).isRequired,
  max: React.PropTypes.number,
  name: React.PropTypes.string,
};

export default ProgressBar;
