/**
 * Created by farhad on 10.06.16.
 */

import _ from 'lodash';
import React from 'react';
import moment from 'moment';
import { connect } from 'react-redux';
import { createSelector } from 'reselect';
import * as BS from 'react-bootstrap';
import d3 from 'd3';

import Widget from '../components/Widget';
import { fetchSwarmTempestSummary } from '../actions';
import ProgressBar from '../components/ProgressBar';
import NVD3Chart from '../../reports/base/NVD3Chart';

class BvtSwarmTempestSummary extends React.Component {

  static propTypes = {
    milestone: React.PropTypes.string.isRequired,
    isFetching: React.PropTypes.bool.isRequired,
    data: React.PropTypes.object.isRequired,
  };

  componentWillMount() {
    this.props.fetchSwarmTempestSummary(this.props.milestone);
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.milestone !== this.props.milestone) {
      this.props.fetchSwarmTempestSummary(nextProps.milestone);
    }
  }

  renderWidgetBody() {
    const data = this.props.data;
    const swarm = this.props.data.swarm_data[0];
    const tempest_data = _(this.props.data.tempest_data).groupBy('date').toPairs().maxBy(0)[1];
    const bvt_data = data.bvt_data;
    const agg = {};
    _(tempest_data).map('status_counts').each(
      obj => _.mergeWith(agg, obj, (aggCnt, cnt) => (aggCnt || 0) + cnt));
    const tempest = {
      date: tempest_data[0].date,
      status_counts: agg,
      plan_name: tempest_data[0].plan_name,
    };

    const tempestCounts = tempest.status_counts || {};
    const tempestTotal = _(tempestCounts).values().sum();

    const swarmCounts = swarm.status_counts || {};
    const swarmTotal = _(swarm.status_counts).values().sum();

    const bvtChart = () => {
      if (!_.isEmpty(bvt_data)) {
        const bvtData = _.map(bvt_data, 'status_counts');
        const count = bvtData.length;
        const weight = 100 / count;
        const bvtPieData = bvtData.map((val, i) => {
          if (val.untested > 0 && _(val).omit('untested').values().sum() === 0) {
            return {
              label: `BVT${i}`,
              value: weight,
              colorValue: 2,
            };
          } else if (_(val).omit('passed').values().sum() > 0) {
            return {
              label: `BVT${i}`,
              value: weight,
              colorValue: 1,
            };
          }
          return {
            label: `BVT${i}`,
            value: weight,
            colorValue: 0,
          };
        });

        return (
          <NVD3Chart
            type="pieChart"
            height={300}
            datum={bvtPieData}
            configure={(chart) => {
              chart
              .margin({ top: 0, right: 0, bottom: 0, left: 0 })
              .x(d => d.label)
              .y(d => d.value)
              .color((d) => {
                const col = d3.scale.ordinal()
                .domain(
                [2, 1, 0]
                )
                .range(
                ['grey', 'red', 'green']
                );
                return col(d.colorValue);
              })
              .showLabels(false)
              .showLegend(false)
              .donut(true)
              .padAngle(0.05)
              .cornerRadius(true);
            }}
            configureD3={(selection) => {
              const pie = selection.selectAll('g.nv-slice:last-child');
              pie
              .insert('text', 'g')
              .text('BVT')
              .attr('class', 'middleTextBlack')
              .attr('text-anchor', 'middle')
              .attr('dy', '.35em');
            }}
          />
      );
      }
    };

    return (
      <div className="container-fluid">
        <BS.Row>
          <BS.Col md={3} style={{ paddingLeft: 0, paddingRight: 0, marginTop: '-45px' }}>
            { bvtChart() }
          </BS.Col>
          <BS.Col md={9} style={{ paddingLeft: 0 }}>
            <ProgressBar data={swarmCounts} max={swarmTotal} name="Swarm" />
            <ProgressBar data={tempestCounts} max={tempestTotal} name="Tempest" />
          </BS.Col>
        </BS.Row>
      </div>
    );
  }

  render() {
    let body = null;
    let widgetLabel = null;
    let title = 'Recent BVT/Swarm/Tempest';

    if (this.props.isFetching) {
      body = <h3>Fetching data ... </h3>;
      widgetLabel = `Fetching ... ${moment(this.props.data.date).format('MMM D')}`;
    } else if (_.isEmpty(this.props.data)) {
      body = <h3>No swarm and tempest runs yet</h3>;
      widgetLabel = null;
    } else {
      body = this.renderWidgetBody();
      widgetLabel = `${this.props.data.tempest_data[0].plan_name}
        (${moment(this.props.data.tempest_data[0].date).format('MMM D')})`;
      title = `Recent BVT/Swarm/Tempest ${widgetLabel}`;
    }

    return (
      <Widget
        title={title}
        expandUrl="/report/bvt-swarm"
        fill
      >
        {body}
      </Widget>
    );
  }
}

const lastRecordSelector = createSelector(
  state => state,
  (data) => {
    if (_.isEmpty(data.data)) { return {}; }
    return data.data;
  }
);


function mapStateToProps(state) {
  return {
    isFetching: state.dashboard.bvtSwarmTempestSummary.isFetching,
    data: lastRecordSelector(state.dashboard.bvtSwarmTempestSummary) || {},
  };
}

export default connect(mapStateToProps, {
  fetchSwarmTempestSummary,
})(BvtSwarmTempestSummary);
