/**
 * Created by farhad on 09.06.16.
 */
import _ from 'lodash';
import React from 'react';
import { connect } from 'react-redux';
import { createSelector } from 'reselect';
import * as BS from 'react-bootstrap';
import moment from 'moment';

import ProgressBarLabel from '../components/ProgressBarLabel';
import Widget from '../components/Widget';
import { fetchCasesAutomationSummary } from '../actions';

class CasesAutomationProgressSummary extends React.Component {

  static propTypes = {
    milestone: React.PropTypes.string.isRequired,
    isFetching: React.PropTypes.bool.isRequired,
    data: React.PropTypes.object.isRequired,
  };

  componentWillMount() {
    this.props.fetchCasesAutomationSummary(this.props.milestone);
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.milestone !== this.props.milestone) {
      this.props.fetchCasesAutomationSummary(nextProps.milestone);
    }
  }

  renderWidgetBody() {
    const data = this.props.data;
    const dataTotal = _(data).omit(['not_automate']).values().sum();
    const automatedRate = data.automated / dataTotal;
    const inProgressRate = data.in_progress / dataTotal;
    const manualRate = data.manual / dataTotal;
    const automationProgress = (
      <div style={{ position: 'relative' }}>
        <div style={{ position: 'absolute', left: '50%', transform: 'translateX(-50%)', WebkitTransform: 'translateX(-50%)' }}>
          <ProgressBarLabel
            percentage={`${(automatedRate * 100).toFixed(1)}%`}
            part={data.automated}
            total={dataTotal}
            ending={'cases automated'}
          />
        </div>
        <BS.ProgressBar style={{ height: 100 }}>
          <BS.ProgressBar
            style={{ height: 100 }}
            min={0}
            max={100}
            now={automatedRate * 100}
            bsStyle="success"
          />
          <BS.ProgressBar
            style={{ height: 100 }}
            min={0}
            max={100}
            now={inProgressRate * 100}
            bsClass={'progress-bar progress-bar-auto_in_progress'}
          />
          <BS.ProgressBar
            style={{ height: 100 }}
            min={0}
            max={100}
            now={manualRate * 100}
            bsClass={'progress-bar progress-bar-manual'}
          />
        </BS.ProgressBar>
      </div>
    );
    return (
      <div>
        <BS.Table striped condensed hover>
          <tbody>
            <tr>
              <td>{automationProgress}</td>
            </tr>
          </tbody>
        </BS.Table>
        <BS.Table striped bordered condensed hover>
          <tbody>
            <tr style={{ color: '#E06666' }}>
              <th>Manual</th>
              <td>{data.manual}</td>
            </tr>
            <tr style={{ color: '#4C4C4C' }}>
              <th>In Progress</th>
              <td>{data.in_progress}</td>
            </tr>
            <tr style={{ color: '#69BA69' }}>
              <th>Automated</th>
              <td>{data.automated}</td>
            </tr>
          </tbody>
        </BS.Table>
      </div>
    );
  }

  render() {
    return (
      <Widget
        title="Test cases automation progress"
        expandUrl="/report/automation-progress"
        widgetTitle={'TC automation'}
        widgetLabel={` (${moment().format('MMMM D')})`}
        fill
      >
        {this.props.isFetching ? null : this.renderWidgetBody()}
      </Widget>
    );
  }
}


const groupedCountsSelector = createSelector(
  state => state.data,
  (data) => {
    const res = {};
    _(data).map('type_counts').each(
      obj => _.mergeWith(res, obj, (resCnt, cnt) => (resCnt || 0) + cnt));
    return res;
  }
);

function mapStateToProps(state) {
  return {
    isFetching: state.dashboard.casesAutomationProgressSummary.isFetching,
    data: groupedCountsSelector(state.dashboard.casesAutomationProgressSummary),
  };
}

export default connect(mapStateToProps, {
  fetchCasesAutomationSummary,
})(CasesAutomationProgressSummary);
