/**
 * Created by farhad on 07.06.16.
 */

import _ from 'lodash';
import React from 'react';
import { connect } from 'react-redux';
import ReactGridLayout, { WidthProvider } from 'react-grid-layout';
import Multiselect from 'react-bootstrap-multiselect';

import BugsVerificationSummary from './BugsVerificationSummary';
import CasesAutomationProgressSummary from './CasesAutomationProgressSummary';
import FeatureTestingSummary from './FeatureTestingSummary';
import BvtSwarmTempestSummary from './BvtSwarmTempestSummary';
import AcceptanceSummary from './AcceptanceSummary';
import { selectRelease } from '../actions';


const GridLayout = WidthProvider(ReactGridLayout);

const defaultLayout = [
  { w: 6, h: 7, x: 0, y: 0, i: 'BvtSwarmTempestSummary', moved: false, 'static': false },
  { w: 6, h: 9, x: 0, y: 8, i: 'AcceptanceSummary', moved: false, 'static': false },
  { w: 6, h: 11, x: 6, y: 0, i: 'BugsVerificationSummary', moved: false, 'static': false },
  { w: 6, h: 12, x: 0, y: 11, i: 'CasesAutomationProgressSummary', moved: false, 'static': false },
  { w: 6, h: 12, x: 6, y: 11, i: 'FeatureTestingSummary', moved: false, 'static': false },
];

function getFromLocalStorage(itemName, key) {
  let ls = {};
  if (window.localStorage) {
    try {
      ls = JSON.parse(global.localStorage.getItem(itemName)) || {};
    } catch (e) { /* Ignore*/ }
  }
  const res = ls[key];
  if (_.some(res, (val) => val['h'] === 1)) {
    return defaultLayout;
  }
  return res;
}

function saveToLocalStorage(itemName, key, value) {
  if (window.localStorage) {
    global.localStorage.setItem(itemName, JSON.stringify({
      [key]: value,
    }));
  }
}


class Dashboard extends React.Component {

  static propTypes = {
    release: React.PropTypes.string.isRequired,
  };

  static defaultProps = {
    title: 'Main dashboard',
    localStorageItemName: 'mainDashboard',
  };

  constructor(props) {
    super(props);

    this.state = {
      layout: getFromLocalStorage(
        this.props.localStorageItemName, 'layout') || defaultLayout.slice(),
    };

    this.onLayoutChange = this.onLayoutChange.bind(this);
    this.onReleaseSelect = this.onReleaseSelect.bind(this);
  }

  onLayoutChange(layout) {
    saveToLocalStorage(this.props.localStorageItemName, 'layout', layout);
    if (!_.isEqual(this.state.layout, layout)) {
      this.setState({ layout });
    }
  }

  onReleaseSelect() {
    this.props.selectRelease(this.releaseSelector.$multiselect.find(':selected').val());
  }

  render() {
    const releaseData = _.intersection(
      _.map(window.commonStore.testrail_milestones, 'name'),
      window.commonStore.launchpad_milestones
    ).map(v => ({ value: v, selected: v === this.props.release }));

    const releaseSelector = (
      <Multiselect
        key="release.select"
        ref={(c) => { this.releaseSelector = c; }}
        buttonWidth="200"
        maxHeight={300}
        nonSelectedText="select release"
        allSelectedText="all selected"
        onDropdownHide={this.onReleaseSelect}
        data={releaseData}
      />
    );

    return (
      <div className="dashboard" style={{ marginTop: '-29px' }}>
        <div>
          <label style={{ color: 'black', marginLeft: '11px' }}>
            Release:
          </label>
          {releaseSelector}
        </div>

        <div style={{ minWidth: 1000, minHeight: 500, marginTop: '-19px' }}>
          <GridLayout
            onLayoutChange={this.onLayoutChange}
            layout={this.state.layout}
            draggableHandle=".widgetDraggableArea"
            rowHeight={30}
            cols={12}
          >
            <div key="BugsVerificationSummary">
              <BugsVerificationSummary milestone={this.props.release} />
            </div>
            <div key="CasesAutomationProgressSummary">
              <CasesAutomationProgressSummary milestone={this.props.release} />
            </div>
            <div key="FeatureTestingSummary">
              <FeatureTestingSummary milestone={this.props.release} />
            </div>

            <div key="BvtSwarmTempestSummary">
              <BvtSwarmTempestSummary milestone={this.props.release} />
            </div>
            <div key="AcceptanceSummary">
              <AcceptanceSummary milestone={this.props.release} />
            </div>
          </GridLayout>
        </div>
      </div>
  );
  }
}


function mapStateToProps(state, ownProps) {
  return {
    release: state.dashboard.release,
  };
}

export default connect(mapStateToProps, { selectRelease })(Dashboard);
