/**
 * Created by farhad on 10.06.16.
 */

import _ from 'lodash';
import React from 'react';
import { connect } from 'react-redux';
import * as BS from 'react-bootstrap';
import moment from 'moment';

import { safeDiv } from '../../reports/base/utils';
import ProgressBarLabel from '../components/ProgressBarLabel';
import Widget from '../components/Widget';
import { fetchFeatureTestingSummary } from '../actions';
import ProgressBarTitle from '../components/ProgressBarTitle';


class FeatureTestingSummary extends React.Component {

  static propTypes = {
    milestone: React.PropTypes.string.isRequired,
    isFetching: React.PropTypes.bool.isRequired,
    data: React.PropTypes.object.isRequired,
  };

  componentWillMount() {
    this.props.fetchFeatureTestingSummary(this.props.milestone);
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.milestone !== this.props.milestone) {
      this.props.fetchFeatureTestingSummary(nextProps.milestone);
    }
  }

  renderWidgetBody() {
    const data = this.props.data;
    const total = _(data).values().sum();
    const completed = total - (data.in_progress || 0);
    const passed = data.passed || 0;
    const blocked = data.blocked + data.untested || 0;
    const nonbloked = total - blocked;

    const renderPct = (cnt, color, popover) => (
      <BS.OverlayTrigger
        trigger={['hover', 'focus']}
        placement="bottom"
        overlay={<BS.Popover id={popover}> {popover} </BS.Popover>}
      >
        <BS.Label style={{ backgroundColor: color, fontSize: 13 }}>
          {Math.round(cnt / total * 100)} %
        </BS.Label>
      </BS.OverlayTrigger>
    );

    return (
      <div>
        <div>
          <div style={{ position: 'relative' }}>
            <div style={{ position: 'absolute', left: '50%', transform: 'translateX(-50%)', WebkitTransform: 'translateX(-50%)' }}>
              <ProgressBarLabel
                percentage={`${safeDiv(nonbloked * 100, total).toFixed(1)}%`}
                part={nonbloked}
                total={total}
                ending={'cases executed'}
              />
            </div>
            <ProgressBarTitle name="Completeness" />
            <BS.ProgressBar style={{ height: 100 }}>
              <BS.ProgressBar
                style={{ height: 100 }}
                max={total}
                now={nonbloked}
                bsStyle="success"
              />
              <BS.ProgressBar
                style={{ height: 100 }}
                max={total}
                now={total - nonbloked}
                bsClass={'progress-bar progress-bar-placeholder'}
              />
            </BS.ProgressBar>
          </div>
          <div style={{ position: 'relative' }}>
            <div style={{ position: 'absolute', left: '50%', transform: 'translateX(-50%)', WebkitTransform: 'translateX(-50%)' }}>
              <ProgressBarLabel
                percentage={`${safeDiv(passed * 100, total).toFixed(1)}%`}
                part={passed}
                total={total}
                ending={'cases passed'}
              />
            </div>
            <ProgressBarTitle name="Pass rate" />
            <BS.ProgressBar style={{ height: 100 }}>
              <BS.ProgressBar
                style={{ height: 100 }}
                max={total}
                now={passed}
                bsStyle="success"
              />
              <BS.ProgressBar
                style={{ height: 100 }}
                max={total}
                now={total - passed}
                bsClass={'progress-bar progress-bar-placeholder'}
              />
            </BS.ProgressBar>
          </div>
        </div>
        <BS.Table striped condensed hover>
          <tbody>
            <tr style={{ color: '#9CED9C' }}>
              <th style={{ width: 100 }}>Completed</th>
              <td style={{ width: 50 }}>{completed}</td>
              <td>{renderPct(completed, '#9CED9C', 'Completeness %')}</td>
            </tr>
            <tr style={{ color: '#69BA69' }}>
              <th style={{ width: 100 }}>Passed</th>
              <td style={{ width: 50 }}>{passed}</td>
              <td>{renderPct(passed, '#69BA69', 'Success Rate %')}</td>
            </tr>
            <tr style={{ color: '#A0A0A0' }}>
              <th>Blocked</th>
              <td>{blocked}</td>
              <td>{renderPct(blocked, '#A0A0A0', 'Blocked %')}</td>
            </tr>
            <tr style={{ color: '#4C4C4C' }} >
              <th>Total</th>
              <td>{total}</td>
              <td>{}</td>
            </tr>
          </tbody>
        </BS.Table>
      </div>
    );
  }

  render() {
    let body;

    if (this.props.isFetching) {
      body = <h3>Fetching data ... </h3>;
    } else if (_.isEmpty(this.props.data)) {
      body = <h3>No data</h3>;
    } else {
      body = this.renderWidgetBody();
    }

    return (
      <Widget
        title="Feature testing"
        expandUrl="/report/feature-testing-progress"
        widgetTitle={'Feature testing'}
        widgetLabel={`(${moment().format('MMMM D')})`}
        fill
      >
        {body}
      </Widget>
    );
  }
}


function mapStateToProps(state) {
  return {
    isFetching: state.dashboard.featureTestingSummary.isFetching,
    data: state.dashboard.featureTestingSummary.data || {},
  };
}

export default connect(mapStateToProps, {
  fetchFeatureTestingSummary,
})(FeatureTestingSummary);
