/**
 * Created by farhad on 20.02.16.
 */
import _ from 'lodash';
import ReportBox from './base/ReportBox';
import consts from '../consts';


const GRID_DATA_URL = '/api/v1.0/testrail/case_complexities';


const columnDefs = [
  {
    headerName: 'QA Team',
    field: 'qa_team_name',
    rowGroupIndex: 0,
    cellRenderer: {
      renderer: 'group',
    },
    valueGetter(params) {
      return commonStore.getTestRailQATeamName(
          params.data.qa_team) || consts.testrail.DEFAULT_TEAM_NAME;
    },
    cellStyle: { 'font-weight': 'bold' },
    width: 150,
    pinned: true,
  },
  {
    headerName: 'Milestone',
    field: 'milestone_name',
    filter: 'set',
    width: 110,
    columnGroupShow: 'open',
  },
  {
    headerName: 'Priority',
    field: 'priority_name',
    filter: 'set',
    width: 110,
  },
  {
    headerName: 'Smoke',
    field: 'complexity_smoke',
    filter: 'number',
    aggFunc: 'sum',
    width: 110,
  },
  {
    headerName: 'Core',
    field: 'complexity_core',
    filter: 'number',
    aggFunc: 'sum',
    width: 110,
  },
  {
    headerName: 'Advanced',
    field: 'complexity_advanced',
    filter: 'number',
    aggFunc: 'sum',
    width: 110,
  },
  {
    headerName: 'Other',
    field: 'complexity_other',
    aggFunc: 'sum',
    filter: 'number',
    width: 110,
  },
];

export default class CaseComplexities extends ReportBox {

  static defaultProps = Object.assign({}, ReportBox.defaultProps, {
    columnDefs,
    title: 'TestRail: Case Complexities',
    csvExportFilename: 'testrail_case_complexities.csv',
    gridDataUrl: GRID_DATA_URL,
    defaultSortModel: [{ colId: 'qa_team_name', sort: 'asc' }],
  });

  prepareGridData(data) {
    super.prepareGridData(data);

    const mapping = {
      [consts.testrail.Complexity.SMOKE]: 'complexity_smoke',
      [consts.testrail.Complexity.CORE]: 'complexity_core',
      [consts.testrail.Complexity.ADVANCED]: 'complexity_advanced',
    };

    _.each(data, (obj) => {
      obj.total = 0;
      obj.complexity_smoke = 0;
      obj.complexity_core = 0;
      obj.complexity_advanced = 0;
      obj.complexity_other = 0;

      _.each(obj.complexities, (count, complexityId) => {
        const fieldName = mapping[complexityId] || 'complexity_other';
        obj[fieldName] = count;
        obj.total += count;
      });

    });

    return data;
  }
}
