/**
 * Created by farhad on 02.03.16.
 */
import _ from 'lodash';
import ReportBox from './base/ReportBox';

import consts from '../consts';


const columnDefs = [
  {
    headerName: 'QA Team',
    field: 'qa_team_name',
    rowGroupIndex: 0,
    cellRenderer: {
      renderer: 'group',
    },
    valueGetter(params) {
      return window.commonStore.getTestRailQATeamName(
          params.data.qa_team) || consts.testrail.DEFAULT_TEAM_NAME;
    },
    cellStyle: { 'font-weight': 'bold' },
    width: 150,
    pinned: true,
  },
  {
    headerName: 'Milestone',
    field: 'milestone_name',
    filter: 'set',
    width: 100,
  },
  {
    headerName: 'Count',
    field: 'count',
    filter: '',
    aggFunc: 'sum',
    width: 80,
  },
  {
    headerName: 'Id',
    field: 'id',
    filter: 'number',
    width: 80,
  },
  {
    headerName: 'Title',
    field: 'title',
    filter: 'text',
    width: 180,
  },
  {
    headerName: 'Complexity',
    field: 'complexity',
    width: 180,
  },
  {
    headerName: 'Test group',
    field: 'test_group',
    filter: 'text',
    width: 180,
  },
  {
    headerName: 'Section',
    field: 'section.name',
    width: 150,
  },
  {
    headerName: 'Suite',
    children: [
      {
        headerName: 'ID',
        field: 'suite.id',
        filter: 'number',
        width: 50,
      },
      {
        headerName: 'Name',
        field: 'suite.name',
        filter: 'text',
        width: 180,
      },
    ],
  },
  {
    headerName: 'Dates',
    children: [
      {
        headerName: 'Created',
        field: 'created_on',
        filter: 'number',
        width: 100,
      },
      {
        headerName: 'Updated',
        field: 'updated_on',
        filter: 'number',
        width: 100,
      },
    ],
  },
  {
    headerName: 'Estimate',
    children: [
      {
        headerName: 'Estimate',
        field: 'estimate',
      },
      {
        headerName: 'Estimate forecast',
        field: 'estimate_forecast',
      },
    ],
  },
];

export default class Cases extends ReportBox {

  static defaultProps = Object.assign({}, ReportBox.defaultProps, {
    columnDefs,
    title: 'Custom Test Cases Report',
    csvExportFilename: 'testrail_cases.csv',
    gridDataUrl: consts.testrail.CASES_LIST_RESOURCE,
    defaultSortModel: [{ colId: 'qa_team_name', sort: 'asc' }],
  });

  prepareGridData(data) {
    _.each(data, (obj) => {obj.count = 1;});
    return data;
  }
}
