import ReportBox from './base/ReportBox';
import React from 'react';
import moment from 'moment';

const GRID_DATA_URL = '/api/v1.0/diagnostic_info';

const getColumnDefs = () => ([
  {
    headerName: 'Type',
    field: 'sync_type',
  },
  {
    headerName: 'Sync start',
    field: 'sync_start',
    valueGetter: (params) =>
        moment(moment.utc(params.data.sync_start)).local().format('YYYY-MM-DD HH:mm:ss'),
  },
  {
    headerName: 'Sync finish',
    field: 'sync_finish',
    valueGetter: (params) =>
        moment(moment.utc(params.data.sync_finish)).local().format('YYYY-MM-DD HH:mm:ss'),
  },
  {
    headerName: 'Sync duration',
    field: 'sync_duration',
  },
  {
    headerName: 'Result',
    field: 'sync_result',
    cellRenderer: (params) => {
      if (params.data === null) {
        return params.value;
      }
      if (params.data.sync_result === false) {
        return `<i class="alert alert-danger fa fa-exclamation-triangle"
          aria-hidden="true"></i>`;
      }
      return `<i class="alert alert-success fa fa-check-circle"
      aria-hidden="true"></i>`;
    },
  },
  {
    headerName: 'Comments',
    field: 'sync_comment',
    valueGetter: (params) => {
      if (params.data.sync_comment.collected_bugs_count) {
        return `Collected ${params.data.sync_comment.collected_bugs_count} bugs`;
      } else if (params.data.sync_error) {
        return params.data.sync_error;
      }
      return null;
    },
  },
]);

export default class DiagnosticPage extends ReportBox {

  static defaultProps = {
    ...ReportBox.defaultProps,
    columnDefs: getColumnDefs,
    gridDataUrl: GRID_DATA_URL,
    gridHeight: 600,
    title: 'Diagnostic Page',
    lastSyncInfoUrl: '/api/v1.0/lp/sync/last',
    refresh: React.PropTypes.func,
    defaultSortModel: [{ colId: 'sync_finish', sort: 'desc' }],
    agGridOptions: {
      ...ReportBox.defaultProps.agGridOptions,
    },
  };
  renderChartCmp() {
  }
}
