import React from 'react';

import { isEqual, jiraBugLink } from './base/utils';
import ReportBox from './base/ReportBox';
import consts from '../consts';

const getColumnDefs = () => ([
  {
    headerName: 'ID',
    field: 'bug_id',
    width: 130,
    cellRenderer: jiraBugLink,
  },
  {
    headerName: 'Subject',
    field: 'bug_subject',
    width: 500,
  },
  {
    headerName: 'Status',
    field: 'bug_status',
    width: 100,
  },
  {
    headerName: 'Priority',
    field: 'bug_priority',
    width: 100,
  },
  {
    headerName: 'Assignee',
    field: 'bug_assignee',
    width: 100,
  },
  {
    headerName: 'Reporter of the bug',
    field: 'bug_reporter',
    width: 150,
  },
]);
export default class JiraRegression extends ReportBox {

  static defaultProps = {
    ...ReportBox.defaultProps,
    columnDefs: getColumnDefs,
    title: 'Jira: Regression Bugs',
    gridHeight: 400,
    csvExportFilename: 'jira_regression_bugs.csv',
    gridDataUrl: consts.jira.JIRA_REGRESSION,
    lastSyncInfoUrl: '/api/v1.0/lp/sync/last',
    defaultSortModel: [{ colId: 'bug_date', sort: 'asc' }],
    refresh: React.PropTypes.func,
    agGridOptions: {
      ...ReportBox.defaultProps.agGridOptions,
    },
  };

  constructor(props) {
    super(props);
    this.state.selectedMilestones = this.initialConfiguration.selectedMilestones;
    this.state.chartDatum = [];

    this.handleMilestoneSelection = this.handleMilestoneSelection.bind(this);
  }

  componentDidUpdate(prevProps, prevState) {
    const fetchParams = ['selectedMilestones'];
    if (!fetchParams.every(x => isEqual(this.state[x], prevState[x]))) {
      this.fetchGridData();
    }
  }

  getDefaultConfiguration() {
    return {
      ...super.getDefaultConfiguration(),
      selectedMilestones: [window.commonStore.lp_milestone_in_focus.name],
    };
  }

  getConfiguration() {
    return {
      ...super.getConfiguration(),
      selectedMilestones: this.state.selectedMilestones,
    };
  }

  setConfiguration(conf) {
    super.setConfiguration(conf);
    this.setStateIf({ selectedMilestones: conf.selectedMilestones });
  }


  handleMilestoneSelection() {
    const selectedMilestones = [];
    this.milestonesSelector.$multiselect.find(':selected').each((i, elem) => {
      selectedMilestones.push($(elem).val());
    });
    this.setStateIf({ selectedMilestones });
  }

  fetchGridData() {
    this.gridApi.showLoadingOverlay();
    this.setlastSyncDate();

    return $.get(this.props.gridDataUrl, {
      milestone: this.state.selectedMilestones.join(' '),
    })
    .done(data => this.updateGridData(data))
    .always(() => this.gridApi.hideOverlay());
  }

  renderChartCmp() {

  }
}
