import React from 'react';

import FailureDriversTable from './base/FailureDriversTable';
import {
  getComponentsSum, lpBugLink,
  failureDriversHooverSwarmBvt, failureDriversHooverTempest,
  failureDriversAddComment, failureDriversAddETA,
  failureDriversPrepareData, failureDriversHooverOther,
} from './base/utils';

const GRID_DATA_URL = '/api/v1.0/testrail/failure_metrics?status=fixed';

const getColumnDefs = () => ([
  {
    headerName: 'Team',
    field: 'team',
    width: 150,
    pinned: true,
    filterParams: {
      newRowsAction: 'keep',
    },
  },
  {
    headerName: 'Assignee',
    field: 'bug_assignee',
    width: 120,
    filterParams: {
      newRowsAction: 'keep',
    },
  },
  {
    headerName: 'ISO',
    field: 'iso_name',
    width: 90,
    hide: true,
  },
  {
    headerName: 'Milestone_id',
    field: 'milestone_id',
    width: 80,
    hide: true,
  },
  {
    headerName: 'LP Milestone',
    field: 'lp_milestone',
    width: 105,
    filterParams: {
      newRowsAction: 'keep',
    },
  },
  {
    headerName: 'Issue description',
    field: 'bug_title',
    cellRenderer: lpBugLink,
    width: 600,
    filterParams: {
      newRowsAction: 'keep',
    },
  },
  {
    headerName: 'Type',
    field: 'bug_type',
    cellClassRules: {
      'rag-red-outer': params => params.value === 'BLOCKER',
    },
    cellStyle: { 'text-align': 'center' },
    cellRenderer: params =>
            `<span class="rag-element">${params.value}</span>`,
    width: 80,
  },
  {
    headerName: 'BVT&SWARM',
    field: 'bug_swarm_bvt',
    cellRenderer: failureDriversHooverSwarmBvt,
    width: 100,
    filter: 'number',
  },
  {
    headerName: 'Tempest',
    field: 'bug_tempest',
    cellRenderer: failureDriversHooverTempest,
    width: 100,
    filter: 'number',
  },
  {
    headerName: 'Other',
    field: 'bug_other',
    cellRenderer: failureDriversHooverOther,
    width: 100,
    filter: 'number',
  },
  {
    headerName: 'Rate',
    field: 'bug_sumrate',
    valueGetter: params => getComponentsSum([
      'bug_swarm_bvt',
      'bug_tempest',
      'bug_other',
    ], params),
    hide: true,
  },
  {
    headerName: 'Status',
    field: 'bug_status',
    width: 100,
  },
  {
    headerName: 'Importance',
    field: 'bug_importance',
    cellClassRules: {
      'bug-critical': params => params.value === 'Critical',
    },
    cellRenderer: params =>
            `<span class="rag-element">${params.value}</span>`,
    width: 100,
  },
  {
    headerName: 'Bug_submitted',
    field: 'bug_reported',
    width: 100,
  },
  {
    headerName: 'Bug_escalated',
    field: 'bug_escalation',
    width: 100,
  },
  {
    headerName: 'ETA',
    field: 'bug_ETA',
    editable: true,
    width: 90,
    newValueHandler: failureDriversAddETA,
  },
  {
    headerName: 'Comments',
    widht: 200,
    field: 'bug_comment',
    editable: true,
    newValueHandler: failureDriversAddComment,
  },
  {
    headerName: 'Bug_fix_committed',
    field: 'bug_fix_committed',
    width: 100,
  },
]);

class LpFailureDriversFixed extends React.Component {

  constructor(props) {
    super(props);

    this.state = {
      data: [],
      isFetching: true,
    };

    this.fetchGridData = this.fetchGridData.bind(this);
  }

  componentDidMount() {
    this.fetchGridData();
  }

  fetchGridData() {
    this.setState({ isFetching: true });
    return $.get(GRID_DATA_URL)
      .done(this.updateGridData.bind(this));
  }

  updateGridData(data) {
    const preparedData = failureDriversPrepareData(data);
    this.setState({ data: preparedData });
    this.setState({ isFetching: false });
  }

  render() {
    return (
      <FailureDriversTable
        rowData={this.state}
        columnDefs={getColumnDefs()}
        refresh={this.fetchGridData}
        title="Failure Drivers - Fixed"
        gridHeight={700}
        defaultSortModel={[
        { colId: 'bug_fix_committed', sort: 'desc' },
        ]}
      />
    );
  }

}

export default LpFailureDriversFixed;
