/**
 * Created by farhad on 22.04.16.
 */
import _ from 'lodash';
import consts from '../../consts/index';
import { getComponentsSum, toFixedCelRenderer, safeDiv } from './utils';


export function updateColDefs(columns, applyFn) {
  return columns.map((col) => {
    if (col.children) {
      return updateColDefs(col.children, applyFn);
    } else {
      return applyFn(col);
    }
  });
}

const colDefDefaults = {
  aggFunc: 'sum',
  filter: 'number',
  filterParams: { newRowsAction: 'keep' },
};


export function makeStatusFieldKey(statusCode) {
  return `cnt_${statusCode}`;
}


const statsDefaultFieldMap = {
  ..._.mapValues(consts.testrail.Statuses, (v, k) => makeStatusFieldKey(v)),
  TOTAL: 'cnt_total',
  TOTAL_FAILED: 'cnt_total_failed',
  PCT_BLOCKED: 'pct_blocked',
  PCT_SUCCESS_RATE: 'pct_success_rate',
  PCT_COMPLETENESS: 'pct_completeness',
};


export { statsDefaultFieldMap };


export default function getStatusColumns(fieldMapping, collapsed = null) {
  const fieldMap = Object.assign({}, statsDefaultFieldMap, fieldMapping);
  const columns = [
    {
      headerName: 'Total',
      field: fieldMap.TOTAL,
      width: 80,
      chart: { nonStackable: true },
      ...colDefDefaults,
    },
    {
      headerName: 'Passed',
      field: fieldMap.PASSED,
      width: 80,
      chart: { color: '#95d96d' },
      ...colDefDefaults,
    },
    {
      headerName: 'Failed',
      headerClass: 'header-center-aligned',
      children: [
        {
          headerName: 'Failed',
          field: fieldMap.FAILED,
          width: 90,
          columnGroupShow: 'open',
          chart: { color: '#f1888f' },
          ...colDefDefaults,
        },
        {
          headerName: 'ProdFailed',
          field: fieldMap.PROD_FAILED,
          columnGroupShow: 'open',
          width: 100,
          chart: { color: '#f1888f' },
          ...colDefDefaults,
        },
        {
          headerName: 'TestFailed',
          field: fieldMap.TEST_FAILED,
          columnGroupShow: 'open',
          width: 100,
          chart: { color: '#d65151' },
          ...colDefDefaults,
        },
        {
          headerName: 'InfraFailed',
          field: fieldMap.INFRA_FAILED,
          columnGroupShow: 'open',
          width: 100,
          chart: { color: '#f22230' },
          ...colDefDefaults,
        },
        {
          headerName: 'TotalFailed',
          field: fieldMap.TOTAL_FAILED,
          valueGetter: params => getComponentsSum([
            fieldMap.FAILED,
            fieldMap.PROD_FAILED,
            fieldMap.TEST_FAILED,
            fieldMap.INFRA_FAILED,
          ], params),
          width: 100,
          filter: 'number',
          filterParams: { newRowsAction: 'keep' },
          columnGroupShow: 'closed',
          chart: { color: '#8b0000' },
        },
      ],
    },
    {
      headerName: 'Skipped',
      field: fieldMap.SKIPPED,
      width: 85,
      chart: { color: '#edee80' },
      ...colDefDefaults,
    },
    {
      headerName: 'Other',
      field: fieldMap.OTHER,
      width: 80,
      chart: { color: '#eb9beb' },
      ...colDefDefaults,
    },
    {
      headerName: 'Blocked',
      field: fieldMap.BLOCKED,
      width: 90,
      chart: { color: '#d0d0d0' },
      ...colDefDefaults,
    },
    {
      headerName: 'In progress',
      field: fieldMap.IN_PROGRESS,
      width: 105,
      chart: { color: '#4ac9a9' },
      ...colDefDefaults,
    },
    {
      headerName: 'Fixed',
      field: fieldMap.FIXED,
      width: 80,
      chart: { color: '#95d96d' },
      ...colDefDefaults,
    },
    {
      headerName: 'Regression',
      field: fieldMap.REGRESSION,
      width: 100,
      chart: { color: '#f1888f' },
      ...colDefDefaults,
    },
    {
      headerName: 'Untested',
      field: fieldMap.UNTESTED,
      width: 100,
      chart: { color: '#000' },
      ...colDefDefaults,
    },
    {
      headerName: '% Blocked',
      field: fieldMap.PCT_BLOCKED,
      valueGetter(params) {
        return 100 * safeDiv(params.getValue(fieldMap.BLOCKED), params.getValue(fieldMap.TOTAL));
      },
      aggFunc: null,
      cellRenderer: toFixedCelRenderer,
      width: 100,
    },
    {
      headerName: 'Completeness',
      field: fieldMap.PCT_COMPLETENESS,
      valueGetter(params) {
        const blocked = params.getValue(fieldMap.BLOCKED) || 0;
        const untested = params.getValue(fieldMap.UNTESTED) || 0;
        const nonBlocked = params.getValue(fieldMap.TOTAL) - (blocked + untested);
        return 100 * safeDiv(nonBlocked, params.getValue(fieldMap.TOTAL));
      },
      aggFunc: null,
      cellRenderer: toFixedCelRenderer,
      width: 125,
    },
    {
      headerName: 'Success Rate',
      field: fieldMap.PCT_SUCCESS_RATE,
      valueGetter(params) {
        return 100 * safeDiv(params.getValue(fieldMap.PASSED), params.getValue(fieldMap.TOTAL));
      },
      aggFunc: null,
      cellRenderer: toFixedCelRenderer,
      width: 125,
    },
  ];

  if (collapsed) {
    _.each(columns, (column) => {
      column.columnGroupShow = 'open';
      if (column.field === fieldMap.TOTAL) {
        column.columnGroupShow = 'closed';
        column.aggFunc = 'sum';
      }
      if (column.field === fieldMap.PCT_SUCCESS_RATE
        || column.field === fieldMap.PCT_COMPLETENESS
        || column.field === fieldMap.PCT_BLOCKED) {
        delete column.columnGroupShow;
      }
    });
  }

  return columns;
}
