/**
 * Created by farhad on 25.05.16.
 */

import React from 'react';
import _ from 'lodash';
import moment from 'moment';
import SimpleRangePicker from '../../components/SimpleRangePicker';
import ReportBox from './ReportBox';
import './multiChartExtended';
import { getDefaultDatesRanges } from './utils';


export default class extends ReportBox {

  static defaultProps = {
    ...ReportBox.defaultProps,
    getDefaultDatesRanges,
  };

  constructor(props) {
    super(props);
    this.state.datesRange = this.initialConfiguration.datesRange.map(d => moment(d));

    this.onDatesRangeChange = this.onDatesRangeChange.bind(this);
  }

  getDefaultConfiguration() {
    const conf = super.getDefaultConfiguration();
    const datesRange = _.toPairs(this.props.getDefaultDatesRanges())[0][1];

    return {
      ...conf,
      datesRange: datesRange.map((d) => d.toJSON()),
    };
  }

  getConfiguration() {
    const conf = super.getConfiguration();
    return {
      ...conf,
      datesRange: this.state.datesRange.map((d) => d.toJSON()),
    };
  }

  setConfiguration(conf) {
    super.setConfiguration(conf);
    if (conf.datesRange) {
      this.setState({ datesRange: conf.datesRange.map(d => moment(d)) });
    }
  }

  onDatesRangeChange(newRange) {
    this.setState({ datesRange: newRange });
  }

  renderDatesRangePicker() {
    return (
      <SimpleRangePicker
        key="dates.pick"
        ranges={this.props.getDefaultDatesRanges()}
        onChange={this.onDatesRangeChange}
        startDate={this.state.datesRange[0]}
        endDate={this.state.datesRange[1]}
      />
    );
  }

  renderToolBarCmp(extraControls) {
    if (!extraControls) {
      extraControls = { main: [this.renderDatesRangePicker()] };
    }
    return super.renderToolBarCmp(extraControls);
  }
}
