import React, { Component } from 'react';
import { Row, Col, ButtonGroup, Button, Panel, Form } from 'react-bootstrap';

import CheckBox from '../../components/CheckBox';
import AgGridExtended from '../../components/AgGridExtended';

const LAST_SYNC_INFO_URL = '/api/v1.0/testrail/sync/last';


const defaultGridOption = (columnDefs) => ({
  // showToolPanel: this.state.showToolPanel,
  columnDefs,
  // rowData: this.state.rowData,
  rowSelection: 'multiple',
  enableSorting: true,
  enableFilter: true,
  enableColResize: true,
  rowHeight: 28,
  groupSuppressAutoColumn: true,
  groupSelectsChildren: true,
  groupUseEntireRow: false,
  agGridOptions: {
    icons: {
      menu: '<i class="fa fa-bars"></i>',
      filter: '<i class="fa fa-filter"></i>',
      sortAscending: '<i class="fa fa-sort-amount-asc"></i>',
      sortDescending: '<i class="fa fa-sort-amount-desc"></i>',
      sortUnSort: '<i class="fa fa-sort"></i>',
      groupExpanded: '<i class="fa fa-minus-square-o"></i>',
      groupContracted: '<i class="fa fa-plus-square-o"></i>',
      columnGroupOpened: '<i class="fa fa-minus-square"/>',
      columnGroupClosed: '<i class="fa fa-plus-square"/>',
      columnVisible: '<i class="fa fa-check-square"></i>',
      columnHidden: '<i class="fa fa-square"></i>',
      columnRemoveFromGroup: '<i class="fa fa-times"></i>',
    },
  },
});

class Grid extends Component {

  static propTypes = {
    defaultSortModel: React.PropTypes.arrayOf(React.PropTypes.object),
  };

  static defaultProps = {
    // csvExportFilename: 'report.csv',
    defaultSortModel: null,
    // gridHeight: 410,
    // chartHeight: 500,
  }

  constructor(props) {
    super(props);

    this.state = {
      rowData: [],
    };

    this.onGridConfChanged = this.onGridConfChanged.bind(this);
    this.onGridReady = this.onGridReady.bind(this);

    this.debouncedUpdateComponent = _.debounce(this.debouncedUpdateComponent.bind(this), 250);
  }

  componentDidMount() {
    this.fetchInitialGridData();
  }

  componentWillMount() {
    console.log('dd Mount');
  }

  onGridReady({ api, columnApi }) {
    this.gridApi = api;
    this.columnApi = columnApi;
    console.log('In Grid Object');

    this.gridApi.addEventListener('modelUpdated', this.onGridConfChanged);
    this.gridApi.addEventListener('columnVisible', this.onGridConfChanged);
    this.gridApi.addEventListener('columnRowGroupChanged', this.onGridConfChanged);
    this.gridApi.addEventListener('afterFilterChanged', this.onGridConfChanged);
    this.gridApi.addEventListener('cellValueChanged', this.onGridConfChanged);
    this.gridApi.addEventListener('columnMoved', this.onGridConfChanged);
    this.gridApi.addEventListener('columnValueChanged', this.onGridConfChanged);
  }

  onGridConfChanged() {
    if (this._suppressGridEvents) {
      return;
    }
    this.debouncedUpdateComponent();
    // this.debouncedSaveConfigurationToUrl();
  }

  debouncedUpdateComponent() {
    this.forceUpdate();
  }


  /**
   * Returns default (JSON serializable) report configuration
   */
  getDefaultConfiguration() {
    return {
      grid_filter: {},
      grid_sort: this.props.defaultSortModel || [],
    };
  }

  /**
   * This method should apply conf object to the report
   */
  setConfiguration(conf) {
    if (this.gridApi) {
      if (conf.grid_filter) {
        this._suppressGridEvents = true;

        // Workaround for set filter. Without it after grid reload filter
        // will be reset even if filterParam.newRowsAction set to 'keep'.
        _.each(conf.grid_filter, (obj, colId) => {
          const colFilterApi = this.gridApi.getFilterApi(colId);
          if (colFilterApi && _.isFunction(colFilterApi.selectNothing)) {
            colFilterApi.selectNothing();
          }
        });

        this.gridApi.setFilterModel(conf.grid_filter);
        this._suppressGridEvents = false;
      }

      if (conf.grid_sort) {
        this._suppressGridEvents = true;
        this.gridApi.setSortModel(conf.grid_sort);
        this._suppressGridEvents = false;
      }
    }
  }

  getConfiguration() {

  }

  onExportCSV() {
    this.gridApi.exportDataAsCsv({
      fileName: this.props.csvExportFilename,
    });
  }

  getChartDatum() {
    // if (this.gridApi && this.columnApi) {
    //   console.log('send');
    //   this.props.getChartDatum(this.gridApi, this.columnApi)
    // }
  }

  fetchInitialGridData() {
    this.gridApi.showLoadingOverlay();
    this.setlastSyncDate();

    return $.get(this.props.initialDataUrl, { ...this.props.initialDataQuery})
      .done(this.updateGridData.bind(this))
      .always(() => { this.gridApi.hideOverlay(); });
  }

  fetchGridData() {
    this.gridApi.showLoadingOverlay();
    this.setlastSyncDate();

    return $.get(this.props.gridDataUrl, { ...this.props.gridDataQuery })
      .done(this.updateGridData.bind(this))
      .always(() => { this.gridApi.hideOverlay(); });
  }

  updateGridData(data) {
    const preparedData = this.props.preparedData(data);
    this.gridApi.setRowData(preparedData);
    this.setState({ rowData: preparedData });


    if (this.props.defaultSortModel) {
      this.gridApi.setSortModel(this.props.defaultSortModel);
    }

  }

  setlastSyncDate() {
    this.setState({ lastSyncDate: 'fetching... ' });

    return $.get(this.props.lastSyncInfoUrl || LAST_SYNC_INFO_URL)
      .done((info) => {
        if (info) {
          const date = new Date(info.started);
          this.setState({ lastSyncDate: moment(date).fromNow() });
        } else {
          this.setState({ lastSyncDate: '[unknown]' });
        }
      })
      .fail(() => { this.setState({ lastSyncDate: 'cannot fetch data' }); });
  }

  render() {
    // this.getChartDatum();
    return (
      <Panel footer={`Updated ${this.state.lastSyncDate}`}>
        <Row>
          <Col md={10}>
            <Form inline role="form">
              <ButtonGroup bsSize="sm" bsStyle="raised">
                <Button bsStyle="info" bsSize="sm" onClick={() => {}}>
                  <i className="fa fa-refresh" /> Refresh
                </Button>
                <Button onClick={() => {}}>
                  <i className="fa fa-download" /> Download as CSV
                </Button>
              </ButtonGroup>
            </Form>
          </Col>
          <Col md={2}>
            <CheckBox
              label=" Tool Panel"
              groupClassName="pull-right"
              onChange={() => {}}
            />
          </Col>
        </Row>
        <AgGridExtended
          style={{ height: this.props.gridHeight }}
          gridOptions={{
            ...defaultGridOption(this.props.columnDefs),
            onGridReady: this.onGridReady,
          }}
        />
      </Panel>
    )
  }
}

export default Grid;
