/**
 * Created by farhad on 25.02.16.
 */
import React from 'react';
import _ from 'lodash';
import d3 from 'd3';
import nv from 'nvd3';


export default class NVD3Chart extends React.Component {

  static propTypes = {
    type: React.PropTypes.string.isRequired,
    configure: React.PropTypes.func,
    containerStyle: React.PropTypes.object,
    onReady: React.PropTypes.func,
    datum: React.PropTypes.array,
    configureD3: React.PropTypes.func,
  };

  constructor(props) {
    super(props);
    this.renderChart = this.renderChart.bind(this);
  }

  componentDidMount() {
    nv.addGraph(this.renderChart);
  }

  componentDidUpdate() {
    this.renderChart();
  }

  componentWillUnmount() {
    if (this.resizeHandler) {
      this.resizeHandler.clear();
    }
  }

  renderChart() {
    if (!this.chart) {
      this.chart = nv.models[this.props.type]();
    }
      // hook for configuring the chart
    if (this.props.configure) {
      this.props.configure(this.chart);
    }

      // Render chart using d3
    this.selection = d3.select(this.nvdChart)
        .datum(this.props.datum)
        .call(this.chart);

    if (this.props.configureD3) {
      this.selection.call(this.props.configureD3);
    }

      // Update the chart if the window size change.
      // Save resizeHandle to remove the resize listener later.
    if (!this.resizeHandler) {
      this.resizeHandler = nv.utils.windowResize(this.chart.update);
    }
    return this.chart;
  }


  render() {
    return (
      <div className="nv-chart" style={this.props.containerStyle} >
        <svg ref={(c) => { this.nvdChart = c; }} {..._.pick(this.props, ['width', 'height'])} />
      </div>
    );
  }
}
