

export default class AutomationProgressAutomatedFilter {

  init(params) {
    this.eGui = document.createElement('div');
    this.eGui.innerHTML =
        '<div>' +
                  '<div>' +
                      '<select class="ag-filter-select" id="qaFilterType">' +
                          '<option value="1">Equals</option>' +
                          '<option value="2">Less than</option>' +
                          '<option value="3">Greater than</option>' +
                      '</select>' +
                  '</div>' +
                  '<div>' +
                      '<input class="ag-filter-filter" id="qaFilterText" type="text" placeholder="Filter..."/>' +
                  '</div>' +
                  '<div class="ag-filter-apply-panel" id="applyPanel">' +
                      '<button type="button" id="applyButton">Apply</button>' +
                  '</div>' +
              '</div>';
    this.rbFilterType = this.eGui.querySelector('#qaFilterType');
    this.rbFilterText = this.eGui.querySelector('#qaFilterText');
    this.filterType = null;
    this.filterText = null;
    this.rbFilterType.addEventListener('change', this.onFilterTypeChange.bind(this));
    this.rbFilterText.addEventListener('change', this.onFilterTextChange.bind(this));
    this.filterActive = false;
    this.filterParams = params.filterParams;
    this.applyActive = this.filterParams && this.filterParams.apply === true;
    this.filterChangedCallback = params.filterChangedCallback;
    this.filterModifiedCallback = params.filterModifiedCallback;
    this.valueGetter = params.valueGetter;
  }

  onFilterTypeChange() {
    this.filterType = parseInt(this.rbFilterType.value, 10);
    this.filterChanged();
  }

  filterChanged() {
    this.filterModifiedCallback();
    if (!this.applyActive) {
      this.filterChangedCallback();
    }
  }
  onFilterTextChange() {
    let filterText = this.rbFilterText.value;
    if (filterText.trim() === '') {
      filterText = null;
    }
    let newFilter;
    if (filterText !== null && filterText !== undefined) {
      newFilter = parseInt(filterText, 10);
    } else {
      newFilter = null;
    }

    if (this.filterText !== newFilter) {
      this.filterText = newFilter;
      this.filterChanged();
    }
  }
  getGui() {
    return this.eGui;
  }
  doesFilterPass(params) {
    if (this.filterText === null) {
      return true;
    }
    const valueAsNumber = params.node.parent.parent.data.automated;
    switch (this.filterType) {
      case 1:
        return valueAsNumber === this.filterText;
      case 2:
        return valueAsNumber < this.filterText;
      case 3:
        return valueAsNumber > this.filterText;
      default:
                  // should never happen
        console.warn(`invalid filter type ${this.filterType}`);
        return false;
    }
  }
  isFilterActive() {
    return this.filterText !== null;
  }
  getApi() {
    return {
      getModel: () => {
        const model = { value: this.filterText };
        return model;
      },
      setModel: (model) => {
        this.rbFilterText.value = model.value;
      },
    };
  }
}
