/**
 * Created by farhad on 07.06.16.
 */

import { createStore, applyMiddleware } from 'redux';
import { CALL_API, apiMiddleware } from 'redux-api-middleware';
import thunk from 'redux-thunk';
import createLogger from 'redux-logger';

import rootReducer from './reducers';


const apiCallDefaultsMiddleware = store => next => action => {
  const callAPI = action[CALL_API];
  if (callAPI === undefined) {
    return next(action);
  }

  if (callAPI.endpoint && callAPI.endpoint.startsWith('/api/v1.0')) {
    Object.assign(callAPI, {
      method: 'GET',
      credentials: 'same-origin',
    }, callAPI);
  }
  return next(action);
};

const middleware = [
  thunk, apiCallDefaultsMiddleware, apiMiddleware,
];


if (process.env.NODE_ENV !== 'production') {
  middleware.push(createLogger());
}


export default function configureStore(preloadedState) {

  const store = createStore(
    rootReducer,
    preloadedState,
    applyMiddleware(...middleware)
  );

  return store;
}
