from functools import wraps

from flask import redirect
from flask import url_for

from qareporting.auth.models import AuthUsers
from qareporting.admin_panel.models import users


def auth_required(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        user = AuthUsers.get_current()
        if not user:
            return redirect(url_for('security.login'))
        return f(*args, **kwargs)
    return decorated


def admin_role_required(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        user = AuthUsers.get_current()
        if not user.passed and 'admin' in [r for r in user.role]:
            return redirect(url_for('admin.login'))
        return f(*args, **kwargs)
    return decorated

