import logging

from mongoengine import *
from flask_security import RoleMixin
from flask_security import UserMixin
from flask import session

log = logging.getLogger(__name__)


class AuthRoles(Document, RoleMixin):
    name = StringField(max_length=80, unique=True)
    description = StringField(max_length=255)

    def __str__(self):
        return self.name


class AuthUsers(DynamicDocument, UserMixin):
    id = StringField()
    email = StringField(max_length=255)
    name = StringField()
    family_name = StringField()
    given_name = StringField()
    picture = StringField()
    hd = StringField()
    verified_email = BooleanField()
    google_token = StringField()
    passed = BooleanField(default=False)
    roles = ListField(ReferenceField(AuthRoles), default=list)

    @classmethod
    def get_current(cls):
        google_token = session.get('google_token')
        if not google_token:
            log.debug('Unauthorized access')
            return None
        try:
            return cls.objects.get(google_token=google_token[0])
        except AuthUsers.DoesNotExist:
            log.warning("User with google_token {0} "
                        "wasn't found in DB".format(google_token[0]))
            return None

    def __str__(self):
        return '{0} ({1})'.format(self.name, self.email)
