import logging

from flask import Blueprint
from flask import abort
from flask import current_app
from flask import flash
from flask import redirect
from flask import session
from flask import url_for

from qareporting.auth.models import AuthUsers
from qareporting.auth.oauth import get_google

log = logging.getLogger(__name__)
auth = Blueprint('auth', __name__, url_prefix='/auth')


@auth.route('/login')
def login():
    callback = url_for('auth.authorized', _external=True)
    return get_google().authorize(callback=callback)


@auth.route('/authorized')
def authorized():
    resp = get_google().authorized_response()
    if resp is None:
        abort(401)

    google_token = resp['access_token']
    session['google_token'] = (google_token, '')

    user_info = get_google().get('userinfo').data
    domain = user_info.get('hd', None)
    if domain != current_app.config['GOOGLE_APP_DOMAIN']:
        flash('Domain is not allowed')
        return redirect(url_for('pages.index'))

    user_info.update({'google_token': google_token, 'passed': False})
    AuthUsers.objects(email=user_info["email"]).update_one(upsert=True,
                                                           **user_info)
    if user_info['email'] in current_app.config['ADMIN_USERS']:
        AuthUsers.objects(email=user_info['email']).modify(
            role=['admin']
        )

    return redirect(url_for('pages.index'))


@auth.route('/logout', methods=['POST'])
def logout():
    session.pop('google_token', None)
    return redirect(url_for('security.login'))


