import re

from flask import current_app as app
from flask_mail import Message

from qareporting.ext import mail
from qareporting.integration.jira.models import JiraFilter, Issue


def get_recipients():
    health_time = re.compile('([8-9]|[1-9]\d|\d{3,})+ day')
    first_filter_name = JiraFilter.objects.order_by(
        'position'
    ).scalar('name').first()

    query = Issue.objects.filter(
        prod_version=first_filter_name,
        issuetype='QA task',
        status__ne='Done',
        health_time=health_time,
    ).scalar('assignee', 'key')

    result = {}
    for recipient in query:
        user, bug_key = recipient
        if user.username in result:
            result[user.username]['bugs'].append(bug_key)
        else:
            result[user.username] = {
                'email': user.email,
                'name': user.display_name,
                'bugs': [bug_key]
            }

    return result.values()


def send_notification(recipient):
    mail.init_app(app)
    msg = Message(
        sender='qa-reporting@mirantis.com',
        recipients=[recipient['email']],
        cc=['tborukhovich@mirantis.com'],
    )

    bugs = []
    for bug in recipient['bugs']:
        bugs.append('<p>'
                    '<a target="_blank" '
                    'href="https://mirantis.jira.com/browse/{}">'.format(bug) +
                    '{}'.format(bug) +
                    '</a></p>')

    msg.html = ("<html>"
                "<body>Hello, {}".format(recipient['name']) +
                "<p>You should update Feature "
                "Health for the following QA Tasks:"
                '<p>{}</p>'.format(''.join(bugs)) +
                "</body>"
                "</html>"
    )
    msg.subject = 'Notification. Update Feature Health'
    mail.send(msg)
