import datetime

from qareporting.integration.launchpad.consts import OPEN_STATUSES
from qareporting.integration.launchpad.consts import STATUS_NEW
from qareporting.integration.testrail import models
from qareporting.resources.failure_drivers_reporting import (
    get_failure_drivers_data)

ACTUAL_STATUSES = OPEN_STATUSES + [STATUS_NEW]

headers = (
    ('team', 'Team'),
    ('bug_assignee', 'Assignee'),
    ('bug_title', 'Issue description'),
    ('type', 'Type'),
    ('swarm_bvt', 'BVT&SWARM'),
    ('tempest', 'Tempest'),
    ('other', 'Other'),
    ('bug_status', 'Status'),
    ('bug_importance', 'Importance'),
    ('bug_reported', 'Bug_submitted'),
    ('bug_escalation', 'Bug_escalation'),
    ('bug_ETA', 'ETA')
)


def blocker_sort(bug):
    is_swarm = bug['rate']['swarm_bvt'] >= 3
    is_tempest = bug['rate']['tempest'] >= 5
    is_blocker_tag = any(tag in bug['tags']
                         for tag in ['swarm-blocker', 'blocker-for-qa'])
    if is_swarm or is_tempest or is_blocker_tag:
        return 1
    return 0


def get_header_table(swarm, tempest, no_swarm=None):

    keys = models.StatusCounts._fields_ordered

    header_table = ('<table border="1" cellspacing="0" cellpadding="0" '
                    'style="border-collapse:collapse;border:none"')

    tempest_total = 0
    tempest_passed = 0

    if tempest:
        for run in tempest:
            tempest_total += sum(
                run.status_counts[key] for key in keys)
            tempest_passed += run.status_counts.passed

    td_header_table_style = ('style="border:solid gray 1.0pt;'
                             'padding:0in 5.4pt 0in 5.4pt"')

    total_status_counts = None
    passed = None

    if no_swarm:
        total_status_counts = 0
        passed = 0
    else:
        total_status_counts = sum(swarm['status_counts'][key] for key in keys)
        passed = swarm['status_counts']['passed']

    if total_status_counts == 0:
        swarm_rate = 0
    else:
        swarm_rate = 100 * (
            passed / total_status_counts)

    if tempest_passed == 0 or tempest_total == 0:
        tempest_rate = 0
    else:
        tempest_rate = 100 * (tempest_passed / tempest_total)

    if no_swarm:
        run_name_row = '<tr><td {1}>Run name</td><td {1}>{0}</td></tr>'.format(
            tempest[0].plan.name,
            td_header_table_style,
        )
        run_date_row = '<tr><td {1}>Run date</td><td {1}>{0}</td></tr>'.format(
            tempest[0].created_on.strftime('%B %d %Y'),
            td_header_table_style,
        )
    else:
        run_name_row = '<tr><td {1}>Run name</td><td {1}>{0}</td></tr>'.format(
            swarm.plan.name,
            td_header_table_style,
        )
        run_date_row = '<tr><td {1}>Run date</td><td {1}>{0}</td></tr>'.format(
            swarm.created_on.strftime('%B %d %Y'),
            td_header_table_style,
        )

    run_pass_rate_row = ('<tr><td {1}>Swarm pass rate</td>'
                         '<td {1}>{0}</td></tr>').format(
        "{:.2f}%".format(swarm_rate),
        td_header_table_style,
    )
    run_total_cases_row = ('<tr><td {1}>Swarm scope (total cases)</td>'
                           '<td {1}>{0}</td></tr>'.format(
                            total_status_counts,
                            td_header_table_style,
                            ))
    tempest_rate_row = ('<tr><td {1}>Tempest pass rate</td>'
                        '<td {1}>{0}</td></tr>').format(
        "{:.2f}%".format(tempest_rate)
        if tempest
        else 'No runs on this snapshot',
        td_header_table_style,
    )
    tempest_scope_row = ('<tr><td {1}>Tempest scope (total cases)</td>'
                         '<td {1}>{0}</td></tr>'.format(
                            tempest_total if tempest
                            else 'No runs on this snapshot',
                            td_header_table_style,
                            ))

    header_table += '<tbody>{}{}{}{}{}{}</tbody></table><p></p>'.format(
        run_name_row,
        run_date_row,
        run_pass_rate_row,
        run_total_cases_row,
        tempest_rate_row,
        tempest_scope_row
    )

    return header_table


def get_table(sorted_bugs):
    result = []

    td_style = ('style="border-top:none;border-left:none;border-bottom:solid '
                'gray 1.0pt;border-right:solid gray 1.0pt;'
                'padding:0in 5.4pt 0in 5.4pt;height:15.0pt"')
    first_column = ('style="border:solid gray 1.0pt;border-top:none;'
                    'padding:0in 5.4pt 0in 5.4pt;height:15.0pt"')
    td_blocker_style = ('style="border-top:none;border-left:none;'
                        'border-bottom:solid gray 1.0pt;'
                        'border-right:solid gray 1.0pt;'
                        'background:#c00000;padding:0in 5.4pt 0in 5.4pt;'
                        'height:15.0pt"')
    text_blocker_style = ('style="font-size:10.0pt;'
                          'font-family: Calibri, sans-serif;color:white"')
    text_risk_style = ('style="font-size:10.0pt;'
                       'font-family: Calibri, sans-serif;color:red"')
    header_style = ('style="border:solid gray 1.0pt;border-left:none;'
                    'background:#8db4e2;padding:0in 5.4pt 0in 5.4pt;'
                    'height:31.5pt"')

    result += ('<table border="0" cellspacing="0" cellpadding="0"'
               ' style="border-collapse:collapse"><tr>')

    message_header = ['<th {}>{}</th>'.format(header_style, value)
                      for x, value in headers]

    result += ''.join(message_header)

    result += '</tr><tr style="height:15.0pt">'

    bugs_len = len(sorted_bugs)

    for index, bug in enumerate(sorted_bugs):
        row = []
        for key, y in headers:
            if key == 'type':
                if blocker_sort(bug) == 1:
                    row.append('<td {}><span {}>BLOCKER</span></td>'.format(
                        td_blocker_style,
                        text_blocker_style,
                    ))
                else:
                    row.append('<td {}><span {}>RISK</span></td>'.format(
                        td_style,
                        text_risk_style,
                    ))
            elif key == 'swarm_bvt':
                row.append('<td {}>{}</td>'.format(td_style, bug['rate'][key]))
            elif key == 'tempest':
                row.append('<td {}>{}</td>'.format(td_style, bug['rate'][key]))
            elif key == 'other':
                row.append('<td {}>{}</td>'.format(td_style, bug['rate'][key]))
            elif key == 'bug_title':
                row.append('<td {}><a target="_blank" '
                           'href="https://launchpad.net/bugs/{}'
                           '">{}</td>'.format(
                            td_style,
                            bug['bug_id'],
                            bug.get(key, '')))
            elif key == 'bug_reported':
                row.append('<td {}>{}</td>'.format(
                    td_style,
                    bug[key].strftime('%Y-%m-%d')))
            elif key == 'bug_escalation':
                row.append('<td {}>{}</td>'.format(
                    td_style,
                    bug[key].strftime('%Y-%m-%d')))
            elif key == 'team':
                row.append('<td {}>{}</td>'.format(
                    first_column,
                    bug.get(key, '')
                ))
            elif key == 'bug_ETA':
                time = bug.get(key, '')
                if time != 'n/a':
                    try:
                        time = datetime.datetime.strptime(
                            time, '%Y-%m-%d'
                        ).strftime('%Y-%m-%d')
                    except ValueError:
                        time = 'Invalid date'
                row.append('<td {}>{}</td>'.format(
                    td_style,
                    time
                ))
            else:
                row.append('<td {}>{}</td>'.format(td_style, bug.get(key, '')))

        if index != bugs_len:
            result.append(''.join(row) + '</tr><tr style="height:15.0pt">')
        else:
            result.append(''.join(row))

    result += '</tr></table>'

    return result


def recent_swarm_stat(milestone_id):
    milestone_name = models.Milestone.objects.filter(
        id=milestone_id
    ).scalar('name')[0]

    recent_swarm = models.Run.objects(
        name__contains='Swarm',
        milestone_id=milestone_id,
    ).order_by('-created_on').first()

    recent_tempest = models.Run.objects(
        run_type=models.Run.RUN_TYPE_TEMPEST_ID,
        milestone_id=milestone_id,
    ).order_by('-created_on').first()

    recent_tempest_data = models.Run.objects(
        run_type=models.Run.RUN_TYPE_TEMPEST_ID,
        plan_id=recent_swarm.plan.pk,
    )

    plan = None
    header_table = None
    if recent_tempest is None:
        plan = recent_swarm.plan
        header_table = get_header_table(recent_swarm, recent_tempest_data)
    elif recent_swarm.plan.created_on >= recent_tempest.plan.created_on:
        plan = recent_swarm.plan
        header_table = get_header_table(recent_swarm, recent_tempest_data)
    else:
        plan = recent_tempest.plan
        recent_tempest_data = models.Run.objects(
            run_type=models.Run.RUN_TYPE_TEMPEST_ID,
            plan_id=plan.pk,
        )
        header_table = get_header_table(
            recent_swarm, recent_tempest_data, no_swarm=True)

    recent_plan_bugs = get_failure_drivers_data(plan_id=plan.pk)
    recent_plan_bug_ids = [bug['bug_id'] for bug in recent_plan_bugs]

    recent_sorted_bugs = sorted(
        [bug for bug in recent_plan_bugs
         if bug['bug_status'] in ACTUAL_STATUSES],
        key=lambda x: (
            blocker_sort(x),
            x['rate']['swarm_bvt'] + x['rate']['tempest'] + x['rate']['other'],
        ),
        reverse=True)

    active_milestone_bugs = get_failure_drivers_data(milestone_id=milestone_id)

    milestone_sorted_bugs = sorted(
        [bug for bug in active_milestone_bugs
         if bug['bug_status'] in ACTUAL_STATUSES and
         bug['bug_id'] not in recent_plan_bug_ids],
        key=lambda x: (
            blocker_sort(x),
            x['rate']['swarm_bvt'] + x['rate']['tempest'] + x['rate']['other'],
        ),
        reverse=True)

    html = '<!DOCTYPE html><head><style type="text/css">' \
           '.im {color: #ff00 !important;}</style></head>'
    html += '<html><body><p>Dear colleagues, </p>'
    html += ('<p>Here is the latest '
             'state of {} QA blockers (and risks).</p>'.format(milestone_name))

    html += header_table

    if len(recent_sorted_bugs) == 0:
        html += '<p>No bugs found in the recent test run</p>'
    else:
        html += ''.join(get_table(recent_sorted_bugs))

    html += '<p>Bugs affecting previous runs yet not fixed</p>'

    html += ''.join(get_table(milestone_sorted_bugs))

    html += '<p>P.S. Here are some useful links: </p>'

    footer_link1 = ('https://docs.google.com/a/mirantis.com/'
                    'spreadsheets/d/1uA1b0jSEophyVw-Q5esOfASww'
                    'WE4GzcHgtjwWV057zo/edit?usp=sharing')
    footer_link2 = 'https://goo.gl/E1GvHR'

    label = '{} QA blockers list as Google Spreadsheet'.format(milestone_name)

    html += ('<ul><li><a href="{}">{}</a></li><li><a href="{}">'
             'ETA and SLO description for blockers</a></li></ul>'.format(
                 footer_link1,
                 label,
                 footer_link2
                 ))

    html += '</body></html>'

    subject = '[QA BLOCKERS] {} ({})'.format(
        plan.name,
        plan.created_on.strftime('%B %d %Y')
    )

    return html, subject
