import logging
import os

from qareporting.utils import MongoEngineJSONEncoder


class Config(object):
    DEBUG = False
    TESTING = False

    # Define the application directory
    PROJECT_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))

    # Secret key for signing cookies
    SECRET_KEY = '<hard to guess string>'

    # Logging
    LOGGING_FORMAT = '[%(asctime)s] - %(name)s - %(levelname)s - %(message)s'
    LOGGING_LEVEL = logging.INFO

    MONGODB_DB = 'qareporting'
    MONGODB_HOST = 'localhost'
    MONGODB_PORT = 27017

    # Caching
    CACHING = {'CACHE_TYPE': 'simple'}
    CACHE_DEFAULT_TIMEOUT = 60 * 60 * 12

    # Serializing
    RESTFUL_JSON = {'cls': MongoEngineJSONEncoder}

    # Fixtures
    FIXTURE_MODULES = [
        'integration.testrail',
    ]
    WEBPACK_MANIFEST_PATH = './../manifest.json'


class ProductionConfig(Config):
    CACHING = {
        'CACHE_TYPE': 'filesystem',
        'CACHE_DIR': '.cache/',
    }
    LOGGING_LOCATION = '/var/log/qareporting/qareporting.log'


class DevelopmentConfig(Config):
    DEBUG = True
    LOGGING_LEVEL = logging.DEBUG
    DEBUG_TB_INTERCEPT_REDIRECTS = False


class TestingConfig(Config):
    TESTING = True

config = {
    'development': 'qareporting.config.DevelopmentConfig',
    'production': 'qareporting.config.ProductionConfig',
    'testing': 'qareporting.config.TestingConfig',
}
