import logging
import requests
from urllib import parse

from flask_script import Command
from flask import current_app


log = logging.getLogger(__name__)


REQUEST_TOKEN_URL = 'https://launchpad.net/+request-token'
ACCESS_TOKEN_URL = 'https://launchpad.net/+access-token'
AUTHORIZE_URL_TEMPLATE = 'https://launchpad.net/+authorize-token?oauth_token={}'
PRINT_MESSAGE_TEMPLATE = (
    'Copy the following to qareport.config:\n'
    'LAUNCHPAD_OAUTH_ACCESS_TOKEN = "{oauth_token}"\n'
    'LAUNCHPAD_OAUTH_ACCESS_TOKEN_SECRET = "{oauth_token_secret}"\n'
)


def parse_token(content):
    return dict(parse.parse_qsl(content))


class GenerateLPCredentials(Command):

    @staticmethod
    def _generate_oauth_request_token():
        params = dict(
            oauth_consumer_key=current_app.config['LAUNCHPAD_CONSUMER_KEY'],
            oauth_signature_method='PLAINTEXT',
            oauth_signature='&',
        )
        response = requests.post(REQUEST_TOKEN_URL, params)
        if response.status_code != 200:
            raise AssertionError(
                "Can't fetch request token: {}".format(response.content))
        return parse_token(response.text)

    @staticmethod
    def _get_oauth_access_token(request_token, request_token_secret):

        params = dict(
            oauth_consumer_key=current_app.config['LAUNCHPAD_CONSUMER_KEY'],
            oauth_signature_method='PLAINTEXT',
            oauth_token=request_token,
            oauth_signature='&{}'.format(request_token_secret)
        )
        response = requests.post(ACCESS_TOKEN_URL, params)
        if response.status_code != 200:
            raise AssertionError(
                "Can't fetch access token: {}".format(response.content))
        return parse_token(response.text)

    def run(self):
        request_token = self._generate_oauth_request_token()

        authorize_url = AUTHORIZE_URL_TEMPLATE.format(
            request_token['oauth_token'])

        print("Open link in browser {}".format(authorize_url))

        prompt = "After authorize type 'done' to continue: "
        while input(prompt).strip() != 'done':
            pass

        access_token = self._get_oauth_access_token(
            request_token['oauth_token'],
            request_token['oauth_token_secret'],
        )

        print(PRINT_MESSAGE_TEMPLATE.format(**access_token))
