import argparse
import logging

from flask_script import Command
from flask_script import Option

from qareporting.integration.launchpad.sync import (
    LaunchpadBugsSync,
    LaunchpadProjectsSync,
    TeamsMappingSync,
    LAST_SYNC,
)
from qareporting.utils import dt_from_iso

log = logging.getLogger(__name__)


def _clean_modified_since(s):
    if s == LAST_SYNC:
        return s
    try:
        return dt_from_iso(s)
    except ValueError:
        msg = "Not a valid date: '{0}'.".format(s)
        raise argparse.ArgumentTypeError(msg)


class SyncLaunchpadCommand(Command):
    option_list = (
        Option(
            '-p', '--projects',
            action='store_true',
            help='Sync projects.',
        ),
        Option(
            '-b', '--bugs',
            action='store_true',
            help='Sync bugs.',
        ),
        Option(
            '-t', '--teams',
            action='store_true',
            help='Sync bugs.',
        ),
        Option(
            '-m', '--modified-since',
            metavar='DATE',
            dest='modified_since',
            required=False,
            default=LAST_SYNC,
            type=_clean_modified_since,
            help='Sync only bugs modified since DATE.',
        ),
        Option(
            '--suppress-history-update',
            dest='suppress_history_update',
            action='store_true',
            help='Do not update BugTaskHistory collection.',
        ),
    )

    def run(self, projects=False, bugs=False, teams=False,
            modified_since=None, suppress_history_update=False):

        if projects:
            proj_sync = LaunchpadProjectsSync()
            proj_sync.run()

        if teams:
            teams_sync = TeamsMappingSync()
            teams_sync.run()

        if bugs:
            bug_sync = LaunchpadBugsSync(
                modified_since=modified_since,
                update_history=not suppress_history_update,
            )
            bug_sync.run()


