from flask_script import Manager
from flask_script import Server
from flask_script import Shell

from qareporting import create_app
from qareporting.commands.general import ClearCacheCommand
from qareporting.commands.general import DropCollection
from qareporting.commands.general import DropDB
from qareporting.commands.general import PopulateDB
from qareporting.commands.general import SendMailCommand
from qareporting.commands.general import SendNotificationCommand
from qareporting.commands.general import CheckUnfinished
from qareporting.commands.general import SendIncompleteBugNotification
from qareporting.ext import db
from qareporting.integration.commands.make_iterations import (
    MakeIterationsCommand
)
from qareporting.integration.jira.commands.sync_jira import SyncJiraCommand
from qareporting.integration.launchpad.commands.lp_credentials import (
    GenerateLPCredentials)
from qareporting.integration.launchpad.commands.sync_launchpad import (
    SyncLaunchpadCommand)
from qareporting.integration.launchpad.commands.update_bug_history import (
    UpdateBugTaskHistoryCommand)
from qareporting.integration.testrail.commands.sync import SyncTestRailCommand

manager = Manager(create_app)

manager.add_option('-f', '--config',
                   dest='config_name',
                   default='development',
                   help='configuration (development, testing or production)',
                   required=False)

manager.add_command('runserver', Server(host='127.0.0.1', port=5000))

manager.add_command('shell', Shell(make_context=lambda: {
    'db': db,
    'app': create_app(),
}))

manager.add_command('clear_cache', ClearCacheCommand())
manager.add_command('drop_db', DropDB())
manager.add_command('send_mail', SendMailCommand())
manager.add_command('drop_collection', DropCollection())
manager.add_command('populate_db', PopulateDB())

manager.add_command('sync_testrail', SyncTestRailCommand())

manager.add_command('generate_lp_credentials', GenerateLPCredentials())
manager.add_command('sync_launchpad', SyncLaunchpadCommand())
manager.add_command('update_bug_task_history', UpdateBugTaskHistoryCommand())

manager.add_command('sync_jira', SyncJiraCommand())

manager.add_command('make_iterations', MakeIterationsCommand())

manager.add_command('send_notifications', SendNotificationCommand())

manager.add_command('check_unfinished', CheckUnfinished)

manager.add_command('send_incomplete', SendIncompleteBugNotification)
