import collections
from flask_restful import fields
from flask_restful import abort
from flask_restful import Resource
import mongoengine

from qareporting.ext import api
from qareporting.resources.decorators import auth_required


class BaseResource(Resource):
    method_decorators = [auth_required]

    @classmethod
    def _abort_404(cls, msg):
        abort(404, code=404, message=msg)

    @classmethod
    def get_or_abort(cls, _id, qs):
        try:
            return qs.get(id=_id)
        except (mongoengine.ValidationError, mongoengine.DoesNotExist):
            cls._abort_404('{0} {1} does not exist'.format(
                qs._document._class_name, _id))


class ErrorResource(BaseResource):
    def get(self, path='/'):
        self._abort_404('Path does not exist')


class MappingField(fields.Raw):

    def __init__(self, mapping, **kwargs):
        """
        :param mapping: Key-value mapping
        :param default: Value to return when key does not exists in mapping.
        """
        super().__init__(**kwargs)
        assert isinstance(mapping, collections.Mapping)
        self.mapping = mapping

    def format(self, value):
        return self.mapping.get(value, self.default)


# Catch-All URL
api.add_resource(ErrorResource, '/', '/<path:path>')
