from qareporting.ext import api
from qareporting.ext import cache
from qareporting.integration.testrail.models import Case
from qareporting.integration.testrail.models import Milestone
from qareporting.integration.testrail.models import Priority
from qareporting.resources.base import BaseResource


class CaseComplexitiesList(BaseResource):

    @cache.memoize()
    def get(self):
        priority_map = dict(Priority.objects.scalar('id', 'name'))
        milestones_map = dict(Milestone.objects.scalar('id', 'name'))

        agg_query = Case.objects.aggregate(
            {'$group': {
                '_id': {
                    'qa_team': '$custom_qa_team',
                    'priority_id': '$priority_id',
                    'milestone_id': '$milestone_id',
                    'case_complexity': '$custom_case_complexity',
                },
                'count': {'$sum': 1},
            }},
            {'$group': {
                '_id': {
                    'qa_team': '$_id.qa_team',
                    'priority_id': '$_id.priority_id',
                    'milestone_id': '$_id.milestone_id',
                },
                'complexities': {
                    '$push': {
                        'case_complexity': '$_id.case_complexity',
                        'count': '$count',
                    },
                },
            }},
            {'$sort': {'_id': 1}},
        )

        result = []
        for raw in agg_query:
            _id = raw['_id']
            complexities = {x.get('case_complexity', 0): x['count']
                            for x in raw['complexities']}
            item = {
                'qa_team': _id.get('qa_team'),
                'priority_name': priority_map.get(_id.get('priority_id')),
                'milestone_name': milestones_map.get(_id.get('milestone_id')),
                'complexities': complexities,
            }
            result.append(item)

        return result


api.add_resource(CaseComplexitiesList, '/testrail/case_complexities')
