from datetime import timedelta

from qareporting.ext import api
from qareporting.integration.models import EventLog
from qareporting.resources.base import BaseResource
from qareporting.utils import dt_now


class DiagnosticInfo(BaseResource):

    def _delta_to_str(self, tdelta):
        date = {'days': tdelta.days}
        date['hours'], rem = divmod(tdelta.seconds, 3600)
        date['minutes'], date['seconds'] = divmod(rem, 60)
        date.update({
            'minutes': str(date['minutes']).zfill(2),
            'seconds': str(date['seconds']).zfill(2),
        })
        return '{hours}:{minutes}:{seconds}'.format(**date)

    def _get_error_message(self, trace):
        return trace.split('File')[1]

    def get(self):
        sync_from = dt_now() - timedelta(days=3)
        event_query = EventLog.objects().filter(
            started__gt=sync_from
        )

        result = []
        for event in event_query:
            res = {
                'sync_type': event['event'],
                'sync_start': event['started'],
                'sync_result': event['success'],
                'sync_comment': event['result']
            }
            if event.finished is None:
                res.update({
                    'sync_finish': dt_now(),
                    'sync_error': event['id']
                })
            else:
                res.update({
                    'sync_finish': event['finished']
                })
            res.update({
                'sync_duration': self._delta_to_str(
                    res['sync_finish'] - res['sync_start']
                )
            })
            res.update({
                'sync_start': res['sync_start'],
                'sync_finish': res['sync_finish']
            })
            if event.error:
                res.update({
                    'sync_error': self._get_error_message(event['error'])
                })
            result.append(res)

        return result


api.add_resource(DiagnosticInfo, '/diagnostic_info')
