from qareporting.ext import api
from qareporting.integration.jira.models import (
    JiraFilter, BugHist, JiraBugsFilter
)
from qareporting.resources.base import BaseResource
from qareporting.integration.models import EventLog


class JiraFilters(BaseResource):
    def get(self):
        filters = JiraFilter.objects.all()
        return [{'name': _filter.name,
                 'id': _filter.filter_id,
                 'position': _filter.position}
                for _filter in filters]


class JiraBugFilters(BaseResource):
    def get(self):
        filters = JiraBugsFilter.objects.all()
        return [{'name': _filter.name,
                 'id': _filter.filter_id,
                 'position': _filter.position}
                for _filter in filters]


class JiraFixVersion(BaseResource):
    def get(self):
        versions = BugHist.objects.all().distinct('fix_versions')
        versions.sort()
        return versions


class JiraPrograms(BaseResource):
    def get(self):
        return list(set(BugHist.objects.all().scalar('program')))


class JiraCompatibleVersion(BaseResource):
    def get(self):
        return BugHist.objects.all().distinct(
            'compatible_versions'
        ) + ['none']


class JiraComponents(BaseResource):
    def get(self):
        return BugHist.objects.all().distinct(
            'components'
        ) + ['none']


class JiraLastSync(BaseResource):
    def get(self):
        return EventLog.get_last(EventLog.EVENT_JIRA_SYNC)


api.add_resource(JiraFilters, '/jira/filter_ids')
api.add_resource(JiraBugFilters, '/jira/bug_filter_ids')
api.add_resource(JiraFixVersion, '/jira/fix_versions')
api.add_resource(JiraPrograms, '/jira/jira_programs')
api.add_resource(JiraCompatibleVersion, '/jira/jira_compatible_versions')
api.add_resource(JiraComponents, '/jira/jira_components')
api.add_resource(JiraLastSync, '/jira/sync/last')
