from collections import defaultdict

from flask_restful import reqparse

from qareporting.ext import api
from qareporting.integration.jira.models import BugHist
from qareporting.resources.base import BaseResource
from qareporting.utils import dt_now


class JiraRegression(BaseResource):
    parser = reqparse.RequestParser()
    parser.add_argument('milestone', default='')

    def __init__(self):
        parser = reqparse.RequestParser()
        parser.add_argument('milestone', default='')

    def get(self):
        target_bugs = tuple(BugHist.objects.filter(
            fix_versions__2__exists=True
        ).distinct('key'))

        bugs_query = BugHist.objects.filter(
            key__in=target_bugs
        ).order_by(
            'version_start'
        ).scalar(
            'key', 'status', 'fix_versions'
        )
        bugs = defaultdict(list)
        for bug in bugs_query:
            bugs[bug[0]].append((bug[1], bug[2]))

        regression_keys = []
        for bug_key, bug_data in bugs.items():
            versions = []
            count = 0
            for item in bug_data:
                if item[0] == 'Done':
                    count += 1
                    if count == 1:
                        versions.extend(item[1])
                if count > 1:
                    if len(versions) < len(item[1]):
                        regression_keys.append(bug_key)

        regression_query = BugHist.objects.filter(
            key__in=regression_keys,
            version_end__gt=dt_now(),
        )
        result = []
        for bug in regression_query:
            result.append({
                'bug_id': bug.key,
                'bug_subject': bug.summary,
                'bug_status': bug.status,
                'bug_priority': bug.priority,
                'bug_assignee': bug.assignee,
                'bug_reporter': bug.reporter
            })

        return result

api.add_resource(JiraRegression, '/jira/regression_bugs')
