import functools
import operator

from qareporting.ext import api
from qareporting.integration.launchpad.models import (
    LaunchpadProject, LpMilestoneInFocus)
from qareporting.integration.models import EventLog
from qareporting.resources.base import BaseResource


class LpLastSync(BaseResource):

    def get(self):
        sync_log = EventLog.get_last(EventLog.EVENT_BUGS_SYNC)

        return {
            'started': (
                sync_log.started if sync_log else None
            )
        }


class LpMilestonesList(BaseResource):

    def get(self):
        projects = LaunchpadProject.objects.all()
        project_milestones = [
            set(m.name for m in project.milestones) for project in projects
        ]
        ret = functools.reduce(operator.or_, project_milestones)
        return sorted(ret, reverse=True, key=lambda m: ('-' not in m, m))


class LpMilestoneInFocusResource(BaseResource):
    def get(self):
        mileston = LpMilestoneInFocus.objects.all().first()
        return {'name': mileston.name}


api.add_resource(LpLastSync, '/lp/sync/last')
api.add_resource(LpMilestonesList, '/lp/milestones')
api.add_resource(LpMilestoneInFocusResource, '/lp/lp_milestone_in_focus')
