const path = require('path');
const webpack = require('webpack');
const ManifestRevisionPlugin = require('manifest-revision-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const WebpackNotifierPlugin = require('webpack-notifier');
const _ = require('lodash');
const pkg = require('./package.json');
require('dotenv').config();


const production = process.env.NODE_ENV === 'production';
const config = {
  devtool: 'eval',
  debug: true,

  entry: {
    vendor: _(pkg.dependencies).keys().without('less', 'font-awesome').value(),
    main: [
      path.resolve(__dirname, 'frontend/index.jsx'),
    ],
  },

  output: {
    path: path.join(process.env.STATIC_PATH),
    publicPath: '/assets/build/',
    filename: '[name][hash].js',
  },

  resolve: {
    modulesDirectories: ['node_modules', 'frontend'],
    extensions: ['', '.js', '.jsx'],
  },

  module: {
    loaders: [
      { test: /\.json$/, loader: 'json-loader' },
      {
        test: /\.jsx?$/,
        exclude: /node_modules/,
        loaders: ['babel-loader'],
      },
      {
        test: /\.css$/,
        loader: ExtractTextPlugin.extract('style', 'css'),
      },
      {
        test: /\.less$/,
        loader: ExtractTextPlugin.extract('style', 'css!less'),
      },
      { test: /\.(png|jpg|gif)$/, loader: 'url-loader?limit=10000' },
      {
        test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        loader: 'url-loader?limit=10000&mimetype=application/font-woff',
      },
      { test: /\.(ttf|eot|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/, loader: 'file-loader' },
    ],
  },
  plugins: [
    new webpack.ContextReplacementPlugin(
      /moment[\/\\]locale$/, /en-gb/
    ),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendor',
      minChunks: Infinity,
    }),
    new ExtractTextPlugin('[name][hash].css', {
      allChunks: true,
      disable: false,
    }),
    new webpack.ProvidePlugin({
      $: 'jquery',
      jQuery: 'jquery',
      'window.jQuery': 'jquery',
    }),
    new webpack.DefinePlugin({
      'process.env.FRONTEND_DEV': JSON.stringify(
        process.env.FRONTEND_DEV === 'true'),
      'process.env.NODE_ENV': JSON.stringify(
        process.env.NODE_ENV || 'development'),
    }),
    new webpack.NoErrorsPlugin(),
    new ManifestRevisionPlugin(path.join('manifest.json'), {
      rootAssetPath: path.join(process.env.STATIC_PATH),
      ignorePaths: ['/styles', '/scripts'],
    }),
  ],
};

if (production) {
  config.devtool = null;
  config.debug = false;

  config.plugins.push(
    new webpack.optimize.DedupePlugin(),
    new webpack.optimize.UglifyJsPlugin({
      compressor: { warnings: false },
      output: { comments: false },
    })
  );
} else {
  config.entry.main.unshift('webpack/hot/only-dev-server');

  config.output.publicPath = `http://localhost:8080${config.output.publicPath}`;

  config.plugins.unshift(
    new WebpackNotifierPlugin({ title: 'Webpack' }),
    new webpack.HotModuleReplacementPlugin()
  );
}

module.exports = config;
